/*****************************************************************************/
/*                                                                           */
/*  HyperPanel OS                       File "stm32m3/src/board_stm32m3.c"   */
/*  Board - IO container data configuration for stm32m3.                     */
/*                                                                           */
/*  Author : JYA/TE/FA                  Last modification : 15/04/2024       */
/*                                                                           */
/*****************************************************************************/

 
/*  Some explanations ========================================================



    1    Purpose of this module
    ___________________________


         This module contains the IO container data configuration, the 
parameters for the VMIO drivers for STM32 Nucleo-L152RE, also named MB1136. 
Data includes const data (parameters) and RAM data (drivers context tables). 
This file contains the following items:

        - Driver system parameters and RAM data

        - Character drivers parameters and RAM data

        - Index tables and data

        - A very limited number of board specific C procedures

    Reference documents for this board are:

        - DS10002 Rev 10 April 2021 STM32L151xE STM32L152xE
          stm-datasheet-stm32l152.pdf

        - RM0038 Rev 18 May 2023 Reference manual 
          stm-rm0038-stm32l152.pdf    

        - DB2196 Rev 18 February 2024 NUCLEO-XXXX*
          db2196-nucleo-l152re.pdf    

    The MCU is:

        - STM32L152RET6 (DB2196 - page 3)
        - Flash 512 Kb
        - RAM 80 Kb
        - Cat.5 (RM0038 page 40)
        - Package LQFP-64
    


    2    User manual
    ________________


    2.1  Contents of the present file
    2.2  Static const data overall schematic
    2.3  System drivers
    2.4  Character drivers
    2.5  Block drivers
    2.6  Volume drivers
    2.7  Protocols
    2.8  Services
    2.9  Interrupt handlers


    2.1  Contents of the present file

         Any board is described with a C structures tree, the root structure is
always of type "Board_link". A same file may contains a number of "Board_link"
structures, each corresponding to a different board, allowing a same binary OS
file to be used with variants of a board.


    2.2  Static const data overall schematic

         The diagram which follows is a copy of the schematic one will find at
"2.9.1  General Overview" of "drv_req1.c". So, for more information, one will
find it in "drv_req1.c".


#DIAGRAM ----------------------------------------------------------------------

===============================================================================
INT            Board_link *                           Board descriptions
num_board      boards[]                               See 2.9.2, drv_req1.c
+--------+     +--------+
|        | -+  |        |
+--------+  |  +--------+
            |  |        |      Board_link
            |  +--------+      +------------+
            +->| ADBOARD| -+-> | board_name |
               +--------+  |   | drv_link   |-----+
               |        |  |   +------------+     |
               +--------+  |   A Board_link       |
               |        |  |   structure          |
               +--------+  |                      |
               |        |  |                      |
               +--------+  |                      |
    Table of addresses of  |                      |
    Board_link structures  |                      |
                           |                      |
   Board_link* +--------+  |                      |
        board  | ADBOARD| -+                      |
               +--------+                         |
                                                  |
                                                  |
   +----------------------------------------------+
===|===========================================================================
   |   Drv_link * []                                 System driver description
   |   +-----------+                                       See 2.9.3 and 2.9.9
   +-> |           |     Sdrv_link          Sdrv_desc            of drv_req1.c
       +-----------+     +-----------+      +----------+
       |           | --> | sdrv_desc | ---> | drv_name |
       +-----------+     | dev_link  | -+   | typdrv   |
       |           |     | nb_dev    |  |   | init     | --> proc C
       +-----------+     +-----------+  |   | end      | --> proc C
       |           |      A Sdrv_link   |   +----------+
       +-----------+      structure     |
       |           |                    |   Dev_link[]
       +-----------+                    |   +----------+      +-----+
       |           |                    +-> | dev_name |      |     | -> proc C
       +-----------+                        | dev_desc | ---->|     | -> proc C
       |           |                        | dev_def  | ---+ |     | -> proc C
       +-----------+                        | drv_def  | -+ | |     | -> proc C
       |           |                        +----------+  | | |     | -> proc C
       +-----------+                        | dev_name +  | | |     | -> proc C
       |           |                        | dev_desc |  | | |     | -> proc C
       +-----------+                        | dev_def  |  | | +-----+
       |           |                        | drv_def  |  | |
       +-----------+                        +----------+  | |
       |           |                        | dev_name |  | | +-----+
       +-----------+                        | dev_desc |  | +>|     |
       |           |                        | dev_def  |  |   |     |
       +-----------+                        | drv_def  |  |   +-----+
       |           |                        +----------+  |
       +-----------+                                      |   +-----+
       |           |                                      +-->|     |
       +-----------+                                          |     |
       |           |                                          +-----+
====== +-----------+ ==========================================================
       |           |                                  Driver/protocol/service/
       +-----------+                                  volume driver description
       |           |                                  See 2.9.4 and 2.9.10    
       +-----------+
       |           |     Drv_link          Drv_desc
       +-----------+     +----------+      +----------+
       |           | --> | drv_desc | ---> | drv_name |
       +-----------+     | dev_link | -+   | typdrv   |
       |           |     | nb_dev   |  |   | initdone | --> Init. flag
       +-----------+     +----------+  |   | init     | --> C proc
       |           |     A Drv_link    |   | autnum   |
       +-----------+     structure     |   | lw0      | --> REQ_ALLOC LW  
       |     0     |                   |   | nblw     | --> Total nb of LW
       +-----------+                   |   | nbiod    | --> Nb of user LW
                                       |   | nbmodd   | --> Nb of inits.
  Table of addresses of                |   | tlcm     | --> Telecom/pipe Itf
  Drv_link structures                  |   | adr      | --> User address
                                       |   +----------+
                                       |
                                       |   Dev_link[]
                                       |   +----------+
                                       +-> | dev_name |      +-----+
                                           | dev_desc | ---->|     | -> proc C
                                           | dev_def  | --+  |     | -> proc C
                                           | drv_def  |   |  |     | -> proc C
                                           +----------+   |  |     | -> proc C
                                           | dev_name |   |  +-----+
                                           | dev_desc |   |
                                           | dev_def  |   |
                                           | drv_def  |   |  +-----+
                                           +----------+   +->|     |
                                           | dev_name |      |     |
                                           | dev_desc |      +-----+
                                           | dev_def  |
                                           | drv_def  |
                                           +----------+

#END --------------------------------------------------------------------------


    2.3  System drivers

         The VMIO driver table is a list, a one dimensionnal array of 
"Sdrv_link" / "Drv_link" structure addresses. With system drivers, the used
structure is always of type "Sdrv_link". For any board, the contents of both the 
"Sdrv_link" and "Drv_link" structures are depending from the hardware and this
is the reason why they are located in the present board configuration file. Each
"Sdrv_link" holds a pointer to one structure of type "Sdrv_desc", which is not
defined in the present file and also holds a pointer to an array of "Dev_link"
structures, which is defined in the present configuration file.

typedef struct                         /+ System driver description          +/
  BEGIN                                /+ ---------------------------------- +/
    const Sdrv_desc *sdrv_desc       ; /+ Driver interfaces address          +/
    const Dev_link  *dev_link        ; /+ Device interfaces address          +/
    UCHAR         nb_dev             ; /+ Number of devices in sdev_link[]   +/
  END             Sdrv_link          ; /+ ---------------------------------- +/

typedef struct                         /+ Character driver description       +/
  BEGIN                                /+ ---------------------------------- +/
    const Drv_desc  *drv_desc        ; /+ Driver interface address           +/
    const Dev_link  *dev_link        ; /+ Devices interfaces list addresses  +/
    UCHAR           nb_dev           ; /+ Number of devices in dev_link[]    +/
    const Pipe_desc *pipe            ; /+ Pipe interface address             +/
  END               Drv_link         ; /+ ---------------------------------- +/


         The "Sdrv_desc" structure is an interface that every system driver
must provide. This structure is made of 4 fields. The address of a character
string, which is the driver name, a "typdrv" integer which value is always 
DRV_SYS (0) for every system driver, the "init" address of a driver 
initialization procedure, and the "end" address of the driver termination
procedure. Each system driver source code file has one own "sdrv_desc"
structure.

typedef struct                         /+ VMIO system driver interface       +/
  BEGIN                                /+ ---------------------------------- +/
    const CHAR  *drv_name            ; /+ Driver name                        +/
    UCHAR       typdrv               ; /+ Always 0 with a system driver      +/
                                       /+ Interface procedures ------------- +/
    IWORD       (*init)()            ; /+ Initialization procedure           +/
    IWORD       (*end )()            ; /+ Termination procedure              +/
  END           Sdrv_desc            ; /+ ---------------------------------- +/


         The "Dev_link" structures are also board dependant and so are defined
within the present file. The "dev_desc" field of each "Dev_link" may point to
data structures which are not defined in the present file but, most of the
time, inside the source file of a low-level driver.

typedef struct                         /+ Device interface                   +/
  BEGIN                                /+ ---------------------------------- +/
    const CHAR    *dev_name          ; /+ System device name (unused)        +/
    const void    *dev_desc          ; /+ Low-level procedure addresses      +/
    const void    *dev_def           ; /+ Low-level parameters addresses     +/
    const void    *drv_def           ; /+ High-level parameters addresses    +/
  END             Dev_link           ; /+ ---------------------------------- +/


    2.4  Character drivers

         The VMIO driver table is a list of "Sdrv_link" and "Drv_link" 
addresses. With character drivers, "Drv_link" structures are used. The contents
of those are dependant from the board and so are defined in the present file.
Each "Drv_link" structure holds one pointer to a "Drv_desc" structure, which is
not defined in the present module but is the source file of the character
driver.

         Each character driver, but also each protocol or service has to 
provide, to export one "Drv_desc" structure, which is its interface. The two
first fields of the "Drv_desc" structure (see "board.h") are same as the 
"Sdrv_desc" ("drv_name" and "typdrv")) but the "Drv_desc" structure is made of
much more fields than "Sdrv_desc". Each C source file which is a character
driver contains and exports its own "Drv_desc" structure.


         The "Dev_link" structures are board dependant and are defined in the
present file. The "dev_desc" field of those is most of the time pointing to
a structure we will find in a low-level (hardware dependant code) module.


    2.5  Block drivers

         There is currently no block driver in the present configuration file.


    2.6  Volume drivers

         There is currently no volume driver, no file system, in the present
confiuration file.


    2.7  Protocols

         There is currently no protocol in the present configuration file.


    2.8  Services

         There is currently no service in the present configuration file.


    2.9  Interrupt handlers

         One will find in "9.1.5 VMIT driver - int_sdrv_link" a table named
"ithand", which contains all the adresses of all the interrupt handlers. The
IRQ handler codes are located in low-level, hardware dependant modules. Within
the "ithand" table, the handler name is followed by two integer's which are the
two input parameters given to the handler procedure when called.



    3    Porting guide
    __________________

         The present file holds mainly data that indeed do not require anny
low-level code. The few procedure at the end of this file do not require any
specific dedicated low-level code.



    4    Include files and external references
    __________________________________________
*/

#include <stdtyp.l>                    /* Definitions et constantes Cojyp.   */
#include <event.l>                     /* IND_DATA                           */
#include <board.h>                     /* Board_link                         */
#include <aut.h>                       /* S_evt                              */
#include <vmiodbg.h>                   /* Note_it                            */
#include <vmio.h>                      /* vmio()                             */
#include <req_lib.h>                   /* Req_ctx                            */
#include <drv.h>                       /* DRV_SYS                            */
#include <drv_ram.h>                   /* Seg_def Part_def                   */
#include <drv_dma.h>                   /* Dma_itproc                         */
#include <drv_asy.h>                   /* Asy_desc                           */
#include <drv_kbd.h>                   /* Kbd_desc                           */
#include <drv_req.h>                   /* Itf_ctx                            */
#include <drv_gpio.h>                  /* Gpio_desc                          */
#include <drv_i2c.h>                   /* I2c_desc                           */
#include <vmk.h>                       /* set_idle                           */
                                       /*                                    */
#include <stm32.h>                     /* DmaSdev                            */
#include <sys_stm32.h>                 /* sdrv_sysm3                         */
#include <tim_stm32.h>                 /* handlerTIM_stm32                   */
#include <dma_stm32.h>                 /* Ref_dma32                          */
#include <asy_stm32.h>                 /* handlerASY_stm32                   */
#include <gpio_stm32.h>                /* handlerGPIO_stm32                  */
#include <i2c_stm32.h>                 /* handlerevI2C_stm32                 */
#include <signal_vmit.h>               /* GPIO_CPU Ref_nvic                  */

#define STM32L15xxE                    /* Needed by <defs_stm32.h>           */
#include <defs_stm32.h>                /* XYZ_IRD defines'                   */

#define BOARD_XYZ_DATA_C               /* Driver space/data                  */

/*
    5    Inner workings of the module
    _________________________________



    6    Exported (public) variables
    ________________________________

         6.1  Board_link structure - boards
         6.2  preinit program for "boot_stm32.s" - preinit_empty  
         6.3  Timer numbers - sys_tim and flt_tim
         6.4  Timers reference table - ref_tim32
         6.5  DMA channels/streams reference table - ref_dma32
         6.6  Debug table for vmint.c - tnote_it[]
         6.7  Debug variable for GDB/IDLE mode - use_gdb

     Some drivers may need global variables we define in the present file. 
So strictly, those are exported (public) variables. However, we deliberately 
did not choose to put those here, we preferred to put them with the driver
const data in section 9
   

    6.1  "Board_link" structure - boards

    The "main" procedure of "vmk_emb.c" sets a first global variable named
    "num_board", in our case always to 0, and then sets another global variable
    named "board" to the address of "boards[num_board]", so in our case "board"
    is always set to the address of "board_l152"
*/

extern Board_link board_l152         ; /* Forward reference                  */

Board_link *boards[] =                 /* List of "Board_link" addresses     */
  BEGIN                                /* ------------ |MC  |CPU| RAM|Flash| */
    &board_l152                      , /* 0 STM32L152  |L152|32M| 80K| 256k| */
  END                                ; /* ---------------------------------+ */


/*  6.2  preinit program for "boot_stm32.s" - preinit_empty  

    "preinit_empty" is declared in "sys_stm32.c"
*/

const UINT cnt10usec = 39     ; /* See WAIT10US in boot_stm32.c              */

/*  6.3  Timer numbers - sys_tim and flt_tim
*/

INT       sys_tim                    ; /* tim_arm.c                          */
INT       flt_tim                    ; /* tim_stm32.c                        */


/*  6.4  Timers reference table - ref_tim32

    The "ref_tim32[]" table is directly referenced by the "tim_stm32.c" module
    This table is used by the "get_ref_tim_stm32" function from "tim_stm32.c"
    that takes in input a "uname" string character and that returns the
    corresponding "ldev" and "chan" values.

    The STM32L152 has 11 timers. We have 3 differents variants that are called
    "General-purpose timers", "basic timers" and "watchdog timers". We have 7
     general-purpose timers, 2 basic timers and 2 watchdog timers.

    Amongst the "General-purpose" we have one 32 bits, other are 16 bits.

    TIME1    ??     ??
    TIME2    16-bit General purpose timer - autor-reload up/down counter - DMA
    TIME3    16-bit General purpose timer - autor-reload up/down counter - DMA
    TIME4    16-bit General purpose timer - autor-reload up/down counter - DMA
    TIME5    32-bit General purpose timer - autor-reload up/down counter - DMA
    TIME6    16-bit Basic timer           - upcounter                    - DMA
    TIME7    16-bit Basic timer           - upcounter                    - DMA
    TIM8     ??     ??
    TIM9     16-bit General purpose timer - auto-reload up/down  
    TIM10    16-bit General purpose timer - auto-reload upcounter
    TIM11    16-bit General purpose timer - auto-reload upcounter

*/

const Ref_tim32 ref_tim32[] =          /* Timers reference table             */
  BEGIN                                /* ---------------------------------- */
/*  uname        use     ntim    chan     Used by                            */
/*  -----------  ----    ----    ----     ---------------------------------- */
   "SYS_TIM"    ,USYS ,   5  ,  255  , /* tim_arm.c                          */
   "FLT_TIM"    ,USYS ,   4  ,  255  , /* tim_stm32, digital filter timer    */
   "ASY1_TIM"   ,USYS ,   2  ,  255  , /* asy_stm32.c USART2                 */
   HNULL        ,USYS ,   0  ,    0  , /* End of table mark                  */

  END                                ; /* ---------------------------------- */


/*  6.5  DMA channels/streams reference table - ref_dma32
    -----------------------------------------------------

    The "ref_dma32[]" table is directly referenced by the "dma_stm32.c" module
    This table is used by the "get_ref_dma_stm32" function from "dma_stm32",
    which takes on input a "uname" string character and returns the
    corresponding "ldev", "chan" and "strm" values.

    The STM32L152 have two DMA (DMA1 and DMA2). The two DMA controllers have 12
    channels in total (7 for DMA1 and 5 for DMA2). Each comes with 12 streams
    numbered from 0 to 11. Each stream is associated to a specific hardware
    using the "channel" value. But we cannot associate any device with any
    stream, one has to refer to Table 55 "DMA1 requests for each channel" and
    Table 56 "DMA2 requests for each channel" on page 256 of RM0038. For
    example, USART1_TX is DMA=1,channel=4 and there is no other possibility.

    DMA1
    Periph Chan 1    Chan 2    Chan 3    Chan 4    Chan 5    Chan 6    Chan 7
    ADC1   ADC1      -         -         -         -         -         -
    SPI    -         SPI1_RX   SPI1_TX   SPI2_RX   SPI2_TX   -         -
    USART  -         USART3_TX USART3_RX USART1_TX USART1_RX USART2_RX USART2_TX
    I2C    -         -         -         I2C2_TX   I2C2_RX   I2C1_TX   I2C1_RX

    DMA2
    Periph Chan 1    Chan 2    Chan 3    Chan 4    Chan 5
    SPI3   SPI3_RX   SPI3_TX   -         -         -
    UART4  -         -         UART4_RX  -         UART4_TX
    UART5  UART5_TX  UART5_RX  -         -         -
*/

/* DMA parameters ...........................................................*/

const Ref_dma32 ref_dma32[] =          /* DMA streams reference table        */
  BEGIN                                /* ---------------------------------- */
/*  uname             ldev    strm         chan             Used by          */
/*  --------------   ------   ----------   ----------       ---------------- */
   "NO_DMA"        , 0xFF   , 0xFF       , 0xFF       ,  /* No DMA used      */
    HNULL          ,   0    ,    0       ,   0        ,  /* End of table mark*/
  END                                ; /* ---------------------------------- */


/*  6.6  Debug table for vmit.c - tnote_it[]

       To help identify bugs that may occur after a long running time, a number
of debug tables are provided by the various VMIO modules. These tables can be
very large, and there are cases where we want to reduce them to a minimum
(loader) and others where we want to increase them to a maximum (debugging a
problem on a given platform). We therefore define these tables in the various
board files.

*/

const IWORD max_note_it =   5        ; /* Debug table size.                  */
Note_it     tnote_it     [  5 ]      ; /* Debug circular buffer.             */


/*  6.7  Debug variable for GDB/IDLE mode - use_gdb

         When using GDB/OpenOCD, we have a regular polling of the target by the
debugger or OpenOCD: If we slow down the JTAG clock at 100 or 10 KHz, we will
notice that the bicolor GREEN/RED led of Olimex probe is flashing Green/RED,
indicating activity, even if there is no reason to have any polling. When the
CPU becomes IDLE, t seems it does not answers any more to GDB/OpenOCD requests 
and we get on the GDB window: 

         DAP transaction stalled (WAIT) - slowing down

and then using GDB becomes very difficult or impossible. What is sure, we never 
get this message if "set_idle(1)" is not called. Therefore, the "board_init"
procedure uses a "use_gdb" global variable which is set by "gdb.cfg" and 
"board_init" does not call "set_idle" if this global variable is TRUE

*/

extern UINT use_gdb                  ; /* For "board_init".                  */


/*
    7    Internal constants of the module
    _____________________________________
*/

/*
    8    Internal data types of the module
    ______________________________________
*/

IWORD sdrv_empty(IWORD)                                                       ;

/*
    9    Internal (private) variables of the module


    9.1  VMIO system drivers configuration data
         9.1.1  SYSTEM driver - sys_sdrv_link
         9.1.2  BOARD_HP_HIC driver - board_sdrv_link
         9.1.3  RAM_HARD driver - ram_hard_sdrv_link
         9.1.4  RAM_SOFT driver - ram_soft_sdrv_link
         9.1.5  VMIT driver - int_sdrv_link
         9.1.6  TIME driver - tim_sdrv_link
         9.1.7  DMA driver - dma_sdrv_link
         9.1.8  VMIO driver - vmio_sdrv_link
         9.1.9  RESOLVE driver - rslv_sdrv_link
         9.1.10 GPIO driver - gpio_sdrv_link
         9.1.11 I2C driver - i2c_sdrv_link
         9.1.12 ASY driver - asy_sdrv_link

    9.2  VMIO character drivers configuration data
         9.2.1  ASY driver
         9.2.2  GPIO driver
         9.2.3  KBD driver

    9.3  Block drivers definitions
    9.4  Volume drivers definitions
    9.5  Protocols definitions
    9.6  Services definitions

    9.7  VMIO driver list and other global data structures
         9.7.1  VMIO driver list - drv_l152
         9.7.2  Board_link structure - board_l152
         9.7.3  List of Drvi_itf contexts - drv_req_ctx
*/
   
/*  9.1  VMIO system drivers configuration data 
    ===========================================


    9.1.1  SYSTEM Driver - sys_sdrv_link
    ------------------------------------

    This driver configures CPU and bus clocks. The configuration data we
    provide here is made of:

    - "sys_sdrv_link" is the "Sdrv_link" structure, which is pointed to
      by "drv_l152", the VMIO driver list. The "sdrv_desc" field is the
      address of the "sdrv_sysm3" structure of "sys_stm32.c" and the "dev_link"
      fields is the address of the "dev_sys" structure below defined.

    - "dev_sys" is the "Dev_link" structure, which is pointed to by 
      "sys_sdr_link". There is no low-level procedures (dev_desc) but we have
      a set of high level parameters, "dev_def" points to "sysdef".

    - "sysdef" is an array of UINT's which are the CPU and bus clock 
       parameters. Those parameters are used by the "init" procedure of the
       "sdrv_desc" (here sdrv_sysm3) structure, in our case the "init" function
       is "init_sysm3" (see "sys_stm32.c), which tunes SYSCLK, HCLK, PCLK1 and
       PCLK2 clocks, the internal flash latency, the PLL, prescalers for buses
       AHB, APB1 and APB2.
*/

static const UINT sysdef[] =           /* Parametres du systeme              */
  BEGIN                                /* ---------------------------------- */
    32000000                         , /* SYSCLK  (External 32KHz crystal)   */
    16000000                         , /* HCLK    (Horloge bus AHB)          */
    16000000                         , /* PCLK1   (Horloge bus APB1)         */
    16000000                         , /* PCLK2   (Horloge bus APB2)         */
    0x03008007                       , /* RCC_AHBENR  : DMA1&2 (3<<24),      */
                                       /* flash (1<<15), GPIOA,B&C (7<<0)    */
    0x00020000                       , /* RCC_APB1ENR : USART2 (1<<17)  p157 */
    0x00004000                       , /* RCC_APB2ENR : USART1 (1<<14)  p155 */
  END                                ; /* ---------------------------------- */

static const Dev_link dev_sys =        /* SYS_L152 device for SYSTEM         */
  BEGIN                                /* ---------------------------------- */
    "SYS_L152"                       , /* dev_name  Device name              */
    HNULL                            , /* dev_desc  Low-level proc addresses */
    sysdef                           , /* dev_def   High level parameters    */
    HNULL                            , /* drv_def   Low level parameters     */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sys_sdrv_link = /* SYSTEM driver                      */
  BEGIN                                /* ---------------------------------- */
    &sdrv_sysm3                      , /* sdrv_desc Driver interface         */
    &dev_sys                         , /* dev_link  List of devices          */
    1                                , /* nb_dev    Count of devices         */
  END                                ; /* ---------------------------------- */


/*  9.1.2  BOARD_HP_HIC - board_sdrv_link
    -------------------------------------

    This driver executes board-dependant initializations, if any. Here, it
calls the "board_init" procedure, which code is located in the present file,
currently it only calls "set_idle(1)" in order to allow CPU idle mode. The
needed configuration data is :

    - "board_sdrv_link" is the "Sdrv_link" structure, which is pointed to
      by "drv_l152", the VMIO driver list. The "sdrv_desc" field is the
      address of the "sdrv_board" structure and the "dev_link" field is HNULL.

    - "board_sdrv_desc" is the "Sdrv_desc" structure, which points the 
      "board_init" function though its "init" field. This "board_init" 
      procedure is therefore called by "init_hypsys".
*/

static IWORD board_init                    (IWORD);

static const Sdrv_desc board_sdrv_desc=/* MB1136 driver interface            */
  BEGIN                                /* ---------------------------------- */
    "MB1136"                        ,  /* drv_name  Driver name              */
    DRV_SYS                         ,  /* typdrv    Always 0 for system sdrv */
                                       /* Interface procedures ------------- */
    board_init                      ,  /* init      Initialization procedure */
    sdrv_empty                      ,  /* end       Termination procedure    */
  END                                ; /* ---------------------------------- */

static const Sdrv_link board_sdrv_link=/* MB1136 driver                      */
  BEGIN                                /* ---------------------------------- */
    &board_sdrv_desc                ,  /* sdrv_desc  Driver interface        */
    HNULL                           ,  /* dev_link   List of devices         */
    0                               ,  /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.3  RAM_HARD driver - ram_hard_sdrv_link
    -------------------------------------------

    This driver discovers additionnal, external DRAM devices, if any. It resets
those if requested. With the MB1136 board, there is no external DRAM and no RAM
to reset of start: This driver does nothing. However, in order to do nothing,
the RAM_HARD driver needs the following configuration data:

    - "ramh_sdrv_link" is the "Sdrv_link" structure, which is pointed to by
      "drv_l152", the VMIO driver list. The "sdrv_desc" field value is the
      address of the "ram_hard_sdrv_desc" structure from "drv_ram.c". The
      "dev_link" field points to a list of "Seg_def" structures, here "seg_def"
      and "nb_dev" is the size of the "seg_def" list, here 1 because we only
      have one RAM segment.

    - "ramh_dev_link" is a list of one "Dev_link" structure, which is pointed 
      to by "ramh_sdrv_link", the RAM_HARD driver structure. The "drv_def" 
      field points to a list of "Seg_def" structures, here "seg_def". The count
      of structures is given by the "nb_dev" field of "ramh_sdrv_link" (1).

    - "seg_def" is a list of one "Seg_def" structure. This structure holds the
      "adr" address of the RAM segment, its "size" size in bytes, and a "flags"
      value which is currently unused.

    The values we use to initialize the "adr" and "size" fields of "seg_def"
may be surprising, especially the "&lg_buf_part" value which is a size in
bytes. This requires a little explanation. In fact the two "ad_buf_part0" and
"lg_buf_part0" names are symbols generated by the linker when it reads the
"boards/stm32m3/exe/linker.cfg" where one will find:

 ad_buf_part0 = ORIGIN(RAM_MEM_PART)           ; /+ Part 0 low-address.      +/
 lg_buf_part0 = LENGTH(RAM_MEM_PART)           ; /+ Part 0 size in bytes     +/

    For the linker, a symbol is simply a character string which replaces a 32
bits integer value. There is no symbol data typing: both "ad_buf_part0" and 
"lg_buf_part0" are both 32 bits integer, no matter they are an address or a 
length. But for a C program, a symbol is implicitely a variable, so the sole 
name is "the value at address symbol" and "&symbol" is the address of the 
value, so the linker symbol value. So for example if LENGTH(RAM_MEM_PART) is 
512, then in C "lg_buf_part" is the value at address 512 and "&lg_buf_part" is 
equal to 512.

    The "init_hypsys" procedure will call the "ram_hard_init" procedure through
the "init" field of "ram_hard_sdrv_desc". This procedure will call 
"alloc_init_ram" and will copy "seg_def" to the "init RAM" area. Then the
RAM_SOFT driver will read the copy of "seg_def".
*/

extern UINT         ad_buf_part0     ; /* Linker defined symbol              */
extern UINT         lg_buf_part0     ; /* Linker defined symbol              */

static const Seg_def seg_def[1] =      /* List of RAM segment descriptors    */
  BEGIN                                /* ---------------------------------- */
    (UCHAR*)&  ad_buf_part0          , /* adr  Start of segment address      */
    (UINT  )&  lg_buf_part0          , /* size Segment size in bytes         */
    0                                , /* List of option flags               */
  END                                ; /* ---------------------------------- */

static const Dev_link ramh_dev_link[3]=/* List of device descriptors         */
  BEGIN                                /* ---------------------------------- */
    "PART0"                          , /* dev_name System device name        */
    HNULL                            , /* dec_desc Low-level procedure addr  */
    HNULL                            , /* dev_def  Low-level parameters addr */
    &seg_def[0]                      , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link ramh_sdrv_link= /* RAM_HARD driver                    */
  BEGIN                                /* ---------------------------------- */
    &ram_hard_sdrv_desc              , /* sdrv_desc  Driver interface        */
    ramh_dev_link                    , /* dev_link   List of devices         */
    1                                , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.4  RAM_SOFT driver - ram_soft_sdrv_link
    -------------------------------------------

    This driver is responsible for execution the first(s) call(s) to the
"init_mem" procedure, which creates one RAM partition at each call. With the
stm32m3 board, we create partition number 0. To do that, the RAM_SOFT (the
"ram_soft_init" procedure) needs the following configuration data:

    - "rams_sdrv_link" is the "Sdrv_link" structure, which is pointed to by
      "drv_l152", the VMIO driver list. The "sdrv_desc" field value is the
      address of the "ram_soft_sdrv_desc" structure from "drv_ram.c". The
      "dev_link" field points to a "rams_dev_link" list one one "Devlink"
      structure. The "nb_dev" field holds the "dev_link"' list size, here 1.

    - "rams_dev_link" is a list of one "Dev_link" structure, we have one and
      one only "Dev_link" structure per RAM partition to be created. The 
      "dev_desc" and "dev_def" fields are not used and so HNULL. The "drv_def"
      field is pointing to a list of one "Part_def" structure.

    - "part_def[1]" is a list of one "Part_def" structure. For each memory
      partition, we may have a list of possible partition parameters, starting
      with biggest sizes and ending with smaller sizes. In our case, we do not
      have external DRAM devices with variable/unknown capacities, we have only
      one and one only RAM segment (see 9.1.3  RAM_HARD driver) and so we only
      have one "Part_def" structure, with only one RAM segment, with a 5KB 
      minimum requested size. The "part" partition number is 0, the selected
      slicing code is 0 and the "big blocks" code size is 7 (4576 bytes).

*/

INT          vmos_memuid             ; /* User id                            */

static const Part_def part_def[2] =    /* List of RAM partition decriptors   */
  BEGIN                                /* ---------------------------------- */
    0                               ,  /* part        Mem partition # 0..3   */
    0                               ,  /* dec         Slicing code 0..3      */
    7                               ,  /* bigszcd     Big blocks size 1..15  */
    0      , 0xFF,0xFF,0xF          ,  /* segnum[4]   Indexes in "Seg_def[]" */
    1024*5 ,   -1,  -1, -1          ,  /* segminsz[4] Segments minimum sizes */
                                       /* ---------------------------------- */
    0xFF                               /* End of list marker                 */
  END                                ; /* ---------------------------------- */

static const Dev_link rams_dev_link[1]=/* List of device descriptors         */
  BEGIN                                /* ---------------------------------- */
    "PART0"                          , /* dev_name System device name        */
    HNULL                            , /* dev_desc Low-level procedure addr  */
    HNULL                            , /* dev_def  Low-level parameters addr */
    part_def                         , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link rams_sdrv_link= /* RAM_SOFT driver                    */
  BEGIN                                /* ---------------------------------- */
    &ram_soft_sdrv_desc             ,  /* sdrv_desc  Driver interface        */
    rams_dev_link                   ,  /* dev_link   List of devices         */
    1                               ,  /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.5  VMIT driver - int_sdrv_link
    ----------------------------------

    This system driver is responsible to initialize the NVIC interrupt 
controller. One has to be aware that next drivers, such as the I2C, the ASY,..
may then in turn raise interrupts, but not before the NVIC has been initialized.
This VMIT system driver needs the following configuration data:

    - "int_sdrv_link" is the "Sdrv_link" structure, which is pointed to by
      "drv_l152", the VMIO driver list. The "sdrv_desc" field value is the
      address of the "vmit_sdrv_desc" structure from "vmit.c". The "dev_link" 
      field points to a list of one "Dev_link" structures, here "int_dev_link".
      The "nb_dev" field value is 1 because we only have one "Dev_link" 
      structure, one IRQ controller (one NVIC).

    - "vmit_dev_link" is the list of one "Dev_link" structure, corresponding to
      the INT_NVIC device. The "dev_desc" field is pointing to "vmit_desc_nvic"
      which is defined in "vmit.c". The "drv_def" field points to "vmit",
      this driver parameter structure is defined below.

    - "int_def" structure of type "Int_def" is pointed by the "drv_def" field
      of "int_dev_link[0]" and holds the three "itidx/ithead/ithand" table
      addresses needed by "vmit.c".
      
    - "ithand" is the table of IRQ handler addresses. It is an array of 
      "Ithand_t" structures, each entry holds one handler C procedure address,
      the two parameters given to the C procedure and the link to the next
      handler associated with the same IRQ, if any. An end of link is marked by
      the ENDLINK special value.

    - "ithead" is the table wich holds the first handler, the head of link, 
      for the chain of handlers for each IRQ, and the links between handlers
      for one same IRQ. Within this table, heads of IRQ are grouped by same
      priorities, beginning by the lowest 0 priority.

    - "itidx" is the table of IRQ priorities, it is a list of indexes in the
      previous "ithead" table, it indicates the limits of the priority groups.

    With ARM CPU's, there is an "exception table" located at address 0x00000000.
Every IRQ is considered as an exception, meaning we have one vector (4 bytes)
per IRQ. When a specific IRQ is taken in account, the CPU fetches the vector
(a code address) and then continues execution at this address. In our case, all
the entries are holding one same address, which is the address of assembly
written "irq" procedure (see "int_cortex.s"). So whatever IRQ is raised, "irq"
is called. The "irq" procedure retreives the exception number from the cortex
IPSR register, substracts 15 in order to get 0 for Systick and "n+1" for IRQ
"n". Then, "irq" extracts from the "itdef" table, which has been built once by
"init_vector_nvic", a 4 byte integer (IRQ priority + "ithead" index value), and
calls "int_proc" (see "vmit.c") with the 4 byte integer has input parameter.
This parameter, and so all the UINT's of "itdef", are equal to 
"prio + index in ithead << 8". So, one can check that the "itdef" contents is
corresponding to "ithead" and then that the indexes in "ithand" from "ithead"
are correct.

    Devices interrupt numbers for STM32L152xxE Cat.5 will be found on
pages 236/237 of RM0038.

*/

/* Handler numbers ......................................................... */

#define H_VMIO                0        /* Handler number                     */
#define H_VMK_TIME            1        /* Handler number                     */

#define H_ASY1                2        /* Handler number                     */

#define H_IRQ40               3        /* Handler number EXTI15_10           */
#define H_GPIO10              4        /* Handler number GPIO10              */
#define H_GPIO11              5        /* Handler number GPIO11              */
#define H_GPIO12              6        /* Handler number GPIO12              */
#define H_GPIO13              7        /* Handler number GPIO13              */
#define H_GPIO14              8        /* Handler number GPIO14              */
#define H_GPIO15              9        /* Handler number GPIO15              */

#define H_I2C_EV             10        /* Handler number I2C event           */
#define H_I2C_ERR            11        /* Handler number I2C error           */

#define H_TIME               12        /* Handler number                     */
#define H_TRIGTO             13        /* Handler number                     */

#define INT_NBIRQ            14        /* Needed by "vmit.h"                 */
#define NB_SOFTIRQ            4        /* Size of "ref_nvic[]"               */

#include <vmit.h>                      /* Ithand_t                           */

/* Data needed by "int_arm.c" .............................................. */

#define VMIO_IRQ     WWDG_IRQ           /* Equal to entry 0  -  0            */
#define VMK_TIME_IRQ UART1_IRQ          /* Equal to entry 1  - 37            */

const Ref_nvic ref_nvic[NB_SOFTIRQ] =  /* Names/numbers of software IRQ's    */
  BEGIN                                /* ---------------------------------- */
    "VMIO"  , VMIO_IRQ           , 1 , /* Window Watchdog                  0 */
    "LPTICK", VMK_TIME_IRQ       , 1 , /* USART1 global interrupt         37 */
  END                                ; /* ---------------------------------- */

/* Handler table .............................................................

   EXTI10 to EXTI15 are sharing one same IRQ line, and the USER button is 
   connected to the PC13 pin. So we use the EXT15_10_IRQ interrupt. Whith
   "handlerGPIO_stm32", par1 is the EXTn index (0..15) and par2 is unused.

   With "handlerevI2C_stm32" and "handlererrI2C_stm32", par1 is the low level 
   context index, so starting with 0. In our case, we only use context 0. The 
   ST device number is 1 and this value is found in "i2c_devdef[]". The
   "par2" is unused.

   With "handlerASY_stm32", par1 is the low level context index, so. The
   "par2" value is _unused.

   Note : Both ithand and ithead are not in FLASH but in RAM is order to
          reduce IRQ latency.
*/


static       Ithand_t ithand[] =  /* Table of IRQ handlers                   */
  BEGIN                                                 
/*  Link b24..31      Handler address      Par1   Par 2       Index          */
/*  --------------    ------------------   -----  ----------  ---------------*/
/* Priority 1 ...............................................................*/

    SINGLE        , vmio              ,  0    , 0        , /*  0 H_VMIO      */
    SINGLE        , vmk_tim_stm32     ,  1    , 0        , /*  1 H_VMK_TIME  */

/* Priority 2 ...............................................................*/

    SINGLE        , handlerASY_stm32  ,  0    , 0        , /*  2 H_ASY1      */

/* Priority 3 ...............................................................*/

    DISPATCH(6)   , dispatchEXTI_IRQ  , 10    , 15       , /*  3 H_IRQ40     */
    ENDLINK       , handlerGPIO_stm32 , 10    , 0        , /*  4 H_GPIO10    */
    ENDLINK       , handlerGPIO_stm32 , 11    , 0        , /*  5 H_GPIO11    */
    ENDLINK       , handlerGPIO_stm32 , 12    , 0        , /*  6 H_GPIO12    */
    ENDLINK       , handlerGPIO_stm32 , 13    , 0        , /*  7 H_GPIO13    */
    ENDLINK       , handlerGPIO_stm32 , 14    , 0        , /*  8 H_GPIO14    */
    ENDLINK       , handlerGPIO_stm32 , 15    , 0        , /*  9 H_GPIO15    */

/* Priority 4 ...............................................................*/

/* Priority 5 ...............................................................*/

    SINGLE        , handlerevI2C_stm32,  0    , 0        , /* 10 H_I2C_EV    */
    SINGLE        , handlererrI2C_stm32, 0    , 0        , /* 11 H_I2C_ERR   */

/* Priority 6 ...............................................................*/

    NEXT(H_TRIGTO), handlerTIM_armv7m ,  0    , 0        , /* 12 H_TIME      */
    ENDLINK       , vmio_trigto       ,  0    , 0        , /* 13 H_TRIGTO    */

/* Priority 7 ...............................................................*/

    0                                                   /* END OF TABLE      */
  END                                                 ; /* ------------------*/

/* Head of chains of handler for each IRQ .................................. */

static       Ithead_t ithead[] =       /* First handler for each IRQ         */
  BEGIN                                /* ---------------------------------- */

/* Priorite 0 is used for currently unused devices (no activated IRQ) ...... */
#define L_ITPRI0  0

/* Priorite 1 .............................................................. */
#define L_ITPRI1  L_ITPRI0 + 2

   VMIO_IRQ      , 1 , H_VMIO        , /* VMIO software raised IRQ           */
   VMK_TIME_IRQ  , 1 , H_VMK_TIME    , /* VMK time software raised IRQ       */

/* Priorite 2 .............................................................. */
#define L_ITPRI2  L_ITPRI1 + 1

   USART2_IRQ    , 1 , H_ASY1        , /* IRQ38, USART2                      */

/* Priorite 3 .............................................................. */
#define L_ITPRI3  L_ITPRI2 + 1

   EXTI15_10_IRQ , 1 , H_IRQ40       , /* IRQ40 , GPIO 10 to GPIO 15         */

/* Priorite 4 .............................................................. */
#define L_ITPRI4  L_ITPRI3 + 0

/* Priorite 5 .............................................................. */
#define L_ITPRI5  L_ITPRI4 + 2

   I2C1_EV_IRQ   , 1 , H_I2C_EV      , /* IRQ31 , I2C1 event                 */
   I2C1_ER_IRQ   , 1 , H_I2C_ERR     , /* IRQ32 , I2C1 error                 */

/* Priorite 6 .............................................................. */
#define L_ITPRI6  L_ITPRI5 + 1

    -1           , 1 , H_TIME        , /* SysTick, decompteur systeme.       */

/* Priorite 7 .............................................................. */
#define L_ITPRI7  L_ITPRI6 + 0

  END                                ; /* ---------------------------------- */

/* Index in "ithead", priority per priority .................................*/

static const IWORD itidx[] =           /* Priority index in "ithead[]"       */
  BEGIN                                /* ---------------------------------- */
    0        , L_ITPRI0 , L_ITPRI1 ,   /* Priority limits (indexes) in the   */
    L_ITPRI2 , L_ITPRI3 , L_ITPRI4 ,   /* "ithead[]" table                   */
    L_ITPRI5 , L_ITPRI6 , L_ITPRI7 ,   /*                                    */
  END                                ; /* ---------------------------------- */

/* Standard configuration data structures for the INT driver ............... */

static const Vmit_def vmit_def =       /* "drv_def" structure for "vmit.c"   */
  BEGIN                                /* ---------------------------------- */
    itidx                            , /* idx  Index for priority groups     */
    ithead                           , /* head Head of links for each IRQ    */
    ithand                           , /* hand IRQ handler addresses         */
  END                                ; /* ---------------------------------- */

static const Dev_link vmit_dev_link[1]=/* List of device descriptors         */
  BEGIN                                /* ---------------------------------- */
    "INT_NVIC"                       , /* dev_name System device name        */
    &vmit_desc_nvic                  , /* dev_desc Low-level procedure addr  */
    HNULL                            , /* dev_def  Low-level parameters addr */
    &vmit_def                        , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */


static const Sdrv_link vmit_sdrv_link =/* INT driver                         */
  BEGIN                                /* ---------------------------------- */
    &vmit_sdrv_desc                  , /* sdrv_desc  Driver interface        */
    vmit_dev_link                    , /* dev_link   List of devices         */
    1                                , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.6  TIME driver - tim_sdrv_link
    ----------------------------------

    The TIME driver maintains the one millisecond tick and increments all the
time related global variable. It also allows for usec measures. In our case,
we use the "hal/armcom/tim_stm32.c" module as TIME driver. This driver needs 
the following configuration data structures:

    - "tim_sdrv_link" is the "Sdrv_link" structure which is pointed to by the
      VMIO table of drivers, "drv_l152" in our case. The "sdrv_desc" field
      points to "sdrv_tim_stm32" which is found in "hal/stm32/tim_stm32.c", the
      "init" field of this structure is pointing to "init_vmk_tim_stm32", which 
      is therefore the procedure called by "init_hypsys". The "dev_link" field 
      is pointing to a one item list of "Dev_link" structure, here 
      "tim_dev_link" and the "nb_dev" count of "Dev_link" structures is 1.

    - "tim_dev_link(1)" is the list of one "Dev_link" structure, all its field
      are empty because "init_timarmv7m" does not needs any configuration date.

*/

static const Dev_link tim_dev_link[1]= /* List of device descriptors         */
  BEGIN                                /* ---------------------------------- */
    "TIM_L152"                       , /* dev_name System device name        */
    HNULL                            , /* dev_desc Low-level procedure addr  */
    HNULL                            , /* dev_def  Low-level parameters addr */
    HNULL                            , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link tim_sdrv_link = /* Driver TIME                        */
  BEGIN                                /* ---------------------------------- */
    &sdrv_tim_stm32                  , /* sdrv_desc  Driver interface        */
    tim_dev_link                     , /* dev_link   List of devices         */
    1                                , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.7  DMA driver - dma_sdrv_desc
    ---------------------------------

    The DMA driver handles DMA devices (Direct Memory Access). This driver needs
the following configuration data:

    - "dma_sdrv_link" is the "Sdrv_link" structure which is pointed to by the
      "drv_l152" VMIO table of drivers. The "sdrv_desc" field points to 
      "dma_sdrv_desc" which is found in "drv_dma.c", the "init" field of this 
      structure is pointing to "dma_init", which is so the procedure called by 
      "init_hypsys". Here, we use 2 DMA controllers, the "dev_link" field is 
      pointing to a two item list of "Dev_link" structures, here
      "dma_dev_link[2]" and the "nb_dev" count of "Dev_link" structures is 2.

    - "dma_dev_link[]" is the array of 2 "Dev_link" structures, one per DMA
      device. The two "dev_desc" fields are pointing to same "dma_desc_stm32"
      structure which is defined in "dma_stm32.c" and holds the addresses of
      the low-level procedures for STM32 DMA devices. We "dev_def" pointers
      have distinct values, are pointing to distinct "Dma_stm32_def" structures,
      that could be separate structures, but we choose to store those in one
      same "dma_drvdec[2]" array. Those two structures are holdingthe low-level
      configuration data, so data which is only meaningfull to the STM32 code.
      The two "dev_def" fields are pointing to two distinct "Dma_drvdef"
      structures, that we stored in one same "dma_drvdef[2]" array.

    - The "dma_drvdef[2]" array holds the high-level driver configuration data,
      so two structure of "Dma_drvdef" type. This structure type definition
      lays in the "drv_dem.h" include file.
*/

#define NUM_DMA              2         /* Number of DMA devices              */
#define DMA_MAX_CTX          16        /* Number of DMA channel contexts     */
#define DMA_MAX_JOB          8         /* Size of "dma_job[]"                */
#include <drv_dma.h>                   /* dma_ctx[] dma_job[]                */


static const Dma_drvdef dma_drvdef[NUM_DMA] = /* Device definition           */
  BEGIN                                /* ---------------------------------- */
                                       /* DMA1 ----------------------------- */
    1                                , /* Device index for low level driver  */
    8                                , /* Number of channels                 */
    T_DMA_GP                         , /* 0:General Purpose 1:Specialized    */
    HNULL                            , /* Maximum numbers of allowed alloc's */
    0x0000                           , /* Available options bitmaps          */
    16                               , /* Memcpy/DMA usage limit             */
                                       /* DMA2 ----------------------------- */
    2                                , /* Device index for low level driver  */
    8                                , /* Number of channels                 */
    T_DMA_GP                         , /* 0:General Purpose 1:Specialized    */
    HNULL                            , /* Maximum numbers of allowed alloc's */
    0x0000                           , /* Available options bitmaps          */
    16                               , /* Memcpy/DMA usage limit             */
  END                                ; /* ---------------------------------- */

static const Dev_link dma_dev_link[] = /* DMA devices                        */
  BEGIN                                /* ---------------------------------- */
    "DMA1"                           , /* dev_name System device name        */
    &dma_desc_stm32                  , /* dev_desc Low-level procedure addr  */
    &ref_dma32                       , /* dev_def  Low-level parameters addr */
    &dma_drvdef[0]                   , /* drv_def  High-level parameters addr*/
                                       /*                                    */
    "DMA2"                           , /* dev_name System device name        */
    &dma_desc_stm32                  , /* dev_desc Low-level procedure addr  */
    & ref_dma32                      , /* dev_def  Low-level parameters addr */
    &dma_drvdef[1]                   , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link dma_sdrv_link=  /* DMA driver                         */
  BEGIN                                /* ---------------------------------- */
    &dma_sdrv_desc                   , /* sdrv_desc  Driver interface        */
    dma_dev_link                     , /* dev_link   List of devices         */
    NUM_DMA                          , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.8  VMIO driver - vmio_sdrv_link
    -----------------------------------

    The VMIO handles the software raised low priority IRQ. The reason we have a
VMIO system driver is to allow VMIO initialization using "init_hypsys". The
VMIO initialization procedure is "init_vmio" and the address of this procedure 
is held in the "init" field of the "vmio_sdrv_link" structure, which is defined
in "vmio.c". Therefore, "init_hypsys" will call "init_vmio". In the same way,
the VMIO has a "end_vmio" termination procedure, its address is kept in the 
"end" field of "vmio_sdrv_desc" and so "end_hypys" will do an indirect call to
"end_vmio".

*/

static const Sdrv_link vmio_sdrv_link= /* VMIO system driver                 */
  BEGIN                                /* ---------------------------------- */
    &vmio_sdrv_desc                  , /* sdrv_desc  Driver interface        */
    HNULL                            , /* dev_link   List of devices         */
    0                                , /* nb_dev     Count of devices        */
 END                                 ; /* ---------------------------------- */


/*  9.1.9  RESOLVE driver - rslv_sdrv_link
    --------------------------------------

    This system driver does not drivers any hardware, be is resolving automaton
names to automaton numbers, which is needed to initialize character drivers:
When a VMK entity needs to send events to VMIO entities, which may be located
in another computer, the distant VMIO number is not known to VMK. Therefore,
any VMIO includes a RESOLVE automaton which receives the REQ_INIT events, the
attached data buffer holds the name. The AUT_RESOLVE scans the VMIO list of
drivers, finds the corresponding driver and so the associated VMIO automaton
and send re-sends the REQ_INIT to that autoamton. The driver/protocol/servide
VMIO automaton will send the RESP_INIT directly to the original REQ_INIT sender
and the "saut" field of RESP_INIT holds the VMIO automaton number. This
"software only" driver needs no configuration data but a "Sdrv_link" structure
which points to "rslv_sdrv_desc", defined in "drv_rslv.c"
*/

static const Sdrv_link rslv_sdrv_link= /* RESOLVE driver                     */
  BEGIN                                /* ---------------------------------- */
    &rslv_sdrv_desc                  , /* sdrv_desc  Driver interface        */
    HNULL                            , /* dev_link   List of devices         */
    0                                , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.10 GPIO driver - gpio_sdrv_link
    -----------------------------------

    The GPIO system driver handles all the MCU GPIO's, which are grouped by
"banks" of 8 GPIOs each, each bank beeing named by ST with one letter ranging 
from A to H. But in our case, our GPIO drivers provides a model with which there
is only ONE GPIO device, with ALL the GPIO's beeing sub-channels on this single
device. The GPIO system driver needs the following configuration data:

    - "gpio_sdrv_link" is the "Sdrv_link" structure which is pointed to by the
      VMIO table of drivers, here "drv_l152". The "sdrv_desc" field points to 
      the "gpio_sdrv_desc" structure which is found in "drv_gpio.c", the "init"
      field of this structure is pointing to "gpio_init", which is therefore the
      procedure called by "init_hypsys". The "dev_link" field is pointing to a 
      one item list of "Dev_link" structure, "gpio_dev_link" and the "nb_dev" 
      count of "Dev_link" structures is 1.

    - "gpio_dev_link(1)" is the list of one "Dev_link" structure, the "dev_desc"
      address of low-level procedure is pointing to "gpio_desc_stm32", which is
      defined in "gpio_stm32.c". The "drv_def" field is pointing to 
      "gpio_pindef", a list of "Gpio_pindef" structures defined below.

    - "gpio_pindef" is an array, a list of "Gpio_pindef" structures.
      are empty because "init_timarmv7m" does not needs any configuration date.
      It has to be noted this array has far less entries than the number of
      MCU's GPIO. This is because by default, all GPIOs are initialized as
      inputs with no pull-up. So, this table only contains the GPIO's for which
      this default configuration is not the one we need.
*/

/* Table of initial configurations that are the "drv_gpio" default one ......

   The pins PA2 and PA3 are connected to ST-LINK MCU (see "UM1724 User       
   manual" (file "STM32NUCLEO_User_manual.pdf"), revision 12 (december       
   2017), chapter "6.8 USART communication").                                

Group     paint       Pin#  ST name    Signal     Alt/Add func     ST device
--------  --------    ----  ---------  ---------  ------------     ---------
JTAG      PA13        46    PA13       JTMS       ----             ----
          PA14        49    PA14       JTCK       ----             ----
          PA15        50    PA15       JTDI       ----             ----
          PB3         55    PB3        JTDO       ----             ----

ASY1      PA2         16    PA2        UART2_TX   ----             ASY1_TX
          PA3         17    PA3        UART2_RX   ----             ASY1_RX

I2C       I2C1 SCL    61    PB8        I2C1_SCL   I2C              I2C1 (CN10-3)
          I2C1 SDA    62    PB9        I2C1_SDA   I2C              I2C1 (CN10-5)

BUT       PC13         2    PC13       GPIO in    ----             BUTUSR

Notes: - When an alternate function is used, so if "func" is not an empty 
         string, field "outcfg" is not used, we put a 0.
*/


const Gpio_pindef gpio_pindef[] =      /* Specific init.                     */
  BEGIN                                /* ---------------------------------- */
/*sname uname     use   func      PU/PD  PP/OD  outcfg speed                 */

/* JTAG signals .............................................................*/

"PA13","JTMS"    ,USYS, "SYS"    , GP_PU, GP_PP, 0  , DEF, /* PA13 JTMS      */
"PA14","JTCK"    ,USYS, "SYS"    , GP_PD, GP_PP, 0  , DEF, /* PA14 JTCK      */
"PA15","JTDI"    ,USYS, "SYS"    , GP_PU, GP_PP, 0  , DEF, /* PA15 JTDI      */
"PB3" ,"JTDO"    ,USYS, "SYS"    , GP_NP, GP_PP, 0  , DEF, /* PB3  JTDO      */

/* ASY1 is USART2 ...........................................................*/

"PA2" ,"ASY1_TX" ,USYS, "UART123", GP_PU, GP_PP, 0  , 0  , /* PA2  TX ST-LINK*/
"PA3" ,"ASY1_RX" ,USYS, "UART123", GP_PU, GP_PP, 0  , 0  , /* PA3  RX ST-LINK*/

/* I2C1 is I2C1 .............................................................*/

"PB8" ,"I2C_SCL" ,USYS, "I2C"    , GP_NP, GP_PP, 0  , 0  , /* PB8 I2C1-SCL   */
"PB9" ,"I2C_SDA" ,USYS, "I2C"    , GP_PU, GP_OD, 0  , 0  , /* PB9 I2C1-SDA   */

/* BUTTON ...................................................................*/

"PC13","USER"    ,UUSR, "GPIO"   , GP_PU, GP_HZ, GP_IN , 0, /* PC13 BUT USER */

 { 0 }                               , /* END OF TABLE                       */
  END                                ; /* ---------------------------------- */

/* GPIO contexts sizes ......................................................*/

#define GPIO_NBCTL             1       /* Only one device.                   */
#define GPIO_NBIOD             1       /* Must be LE than GPIO_NBCTL         */
#define GPIO_NBPIN          8*16       /* Number of pins.                    */
#define GPIO_NBVL      (1+1+8*16)      /* Total number of logical channels.  */

#define GPIO_PKG_NUM           2       /* LQFP64 is 2                        */

#include <drv_gpio.h>                  /* led_nbiod, led_nbvl, led_etat      */

static const Dev_link gpio_dev_link[1]=/* GPIO device                        */
  BEGIN                                /* ---------------------------------- */
    "GPIO"                           , /* dev_name System device name        */
    &gpio_desc_stm32                 , /* dev_desc Low-level procedure addr  */
    HNULL                            , /* dev_def  Low-level parameters addr */
    &gpio_pindef                     , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link gpio_sdrv_link= /* GPIO driver                        */
  BEGIN                                /* ---------------------------------- */
    &gpio_sdrv_desc                  , /* sdrv_desc  Driver interface        */
    gpio_dev_link                    , /* dev_link   List of devices         */
    GPIO_NBCTL                       , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.11 I2C driver - i2c_sdrv_link
    ---------------------------------

    The I2C driver handles all the I2C bus controllers of the MCU, it allows
to send and receive complex I2C frames. The "drv_i2c.c" driver needs the
following configuration data structures:

    - "i2C_sdrv_link" is the "Sdrv_link" structure which is pointed to by the
      VMIO table of drivers, "drv_l152" in our case. The "sdrv_desc" field
      points to "i2c_sdrv_desc" which is found in "drv_i2c.c", the "init" field 
      of this structure is pointing to "i2c_init", which is therefore the 
      procedure called by "init_hypsys". Here, we use only one I2C controller,
      the "dev_link" field is pointing to a one item list of "Dev_link" 
      structure, here "i2c_dev_link[1]" and the "nb_dev" count of "Dev_link" 
      structures is set to 1.

    - "i2c_dev_link[1]" is the list of one "Dev_link" structure. The "dev_desc"
      field points to the "i2c_desc_stm32" from "i2c_stm32.c", which holds the
      low-level procedure addresses. The "dev_def" field points to the low-level
      parameter structure, here "i2c_devdef". And finally, "drv_def" points to
      the high-level driver parameter structures, here "i2c_drvdef".

    - The "i2c_devdef" structure type is "I2c_def_stm32", it helds the low-level
      parameters. For "i2c_stm32.c", the "I2c_def_stm32" structure holds the
      "i2c_num" ST defined controller number and the "fbus" frequency of SCL.

    - The "i2c_drvdef" structure type is "I2c_drvdef", it contains the high-
      level driver parameters. There is only one parameter, which is the delay
      in millisecond between two consecutive bus accesses.
*/

#define I2C_NBCTX            1         /* I2C controllers contexts           */
#define I2C_NBSCTX           1         /* I2C slave devices contexts         */
#define I2C_BCNT             1         /* I2C buffers number                 */
#define I2C_NBNODE           4         /* I2C nodes                          */
#define I2C_SBCNT            1         /* ?                                  */
#define I2C_BBSIZE           1         /* ?                                  */
#define I2C_BBCNT            1         /* ?                                  */
#define NUM_I2C              1         /* Number of used I2Cs                */
#include <drv_i2c.h>                   /* i2c_ctx i2c_etat                   */

#define STM32_I2C            1         /* Number of used STM32 I2Cs          */
#include <i2c_stm32.h>                 /* i2c_ctxstm32                       */

static const
    I2c_def_stm32 i2c_devdef[NUM_I2C]= /* Low level device parameters        */
  BEGIN                                /* ---------------------------------- */
    1                                , /* i2c_num  ST defined controller #   */
    100000                           , /* fbus     I2C bus speed (SCL freq)  */
  END                                ; /* ---------------------------------- */

static const
    I2c_drvdef i2c_drvdef[NUM_I2C]   = /* High level driver parameters       */
  BEGIN                                /* ---------------------------------- */
    "I2C"                            , /* Device name                        */
    0                                , /* Low-level driver context number    */
    0                                , /* Count of slave devices             */
    MODE_MASTER                      , /* OFF MASTER SLAVE TUNMASTER TUNSLAVE*/
    0                                , /* SPI bus speed in KHz               */
    0                                , /* Wait duration between 2 requests   */
    0                                , /* Tunnel polling period in msec      */
    HNULL                            , /* List of slave parameters           */
  END                                ; /* ---------------------------------- */

static const Dev_link i2c_dev_link[] = /* I2C1 device                        */
  BEGIN                                /* ---------------------------------- */
    "I2C1/2"                         , /* dev_name System device name        */
    &i2c_desc_stm32                  , /* dev_desc Low-level procedure addr  */
    i2c_devdef                       , /* dev_def  Low-level parameters addr */
    i2c_drvdef                       , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link i2c_sdrv_link = /* I2C system driver                  */
  BEGIN                                /* ---------------------------------- */
    &i2c_sdrv_desc                   , /* sdrv_desc  Driver interface        */
    i2c_dev_link                     , /* dev_link   List of devices         */
    NUM_I2C                          , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.12 ASY driver - asy_sdrv_link
    ---------------------------------

    The ASY driver handles UART's (Universal Asynchronous Receiver Transmitter).
It has no be noted that ST MCU's are coming with UART's but also USART's which
are Universal Asynchronous and Synchronous Receiver Transmitter, which means
those are UART able. The HyperPanel OS ASY driver only manages UART's and not
the Synchronous mode, USART ST device may only be used in asynchronous mode. 
The HyperPanel ASY system driver needs the following configuration structures:

    - "asy_sdrv_link" is the "Sdrv_link" structure which is pointed to by the
      "drv_l152" VMIO table of drivers. The "sdrv_desc" field points to 
      "asy_sdrv_desc" which is found in "drv_asy.c", the "init" field of this 
      structure is pointing to "asy_init", which is so the procedure called by 
      "init_hypsys". Here, we use 1 ASY controllers, the "dev_link" field is 
      pointing to a two item list of "Dev_link" structures, here "
      asy_dev_link[1]" and the "nb_dev" count of "Dev_link" structures is 1.

    - "asy_dev_link[1]" is the list of three "Dev_link" structure. The   
      "dev_desc" field is pointing to the "asy_desc_stm32" structure from
      "asy_stm32.c", which holds the low-procedure addresses. The "dev_def"
      field is pointing to "asy_def[0]". The "drv_def" field is pointing to
      "asy_drvdef[0]".

    - Each ASY controller needs one structure of low-level (hardware dependant)
      in our case one "Asy_def_stm32" structure (see asy_stm32.h). We have
      ony one entry and we use ST devices 2 (USART2). By default, the USART2
      communication between the target STM32 and ST-LINK MCU is enable
      (UM1724 Rev14 page 25). So for the trace on the console (asy_write()
      procedure), we use USART2 for the "ASY1" ASY context.

    - Each ASY controller needs one structure of high-level driver parameters,
      a "Asy_drvdef" structure for "drv_asy.c". We store it in "asy_drvdef[1]".
      It's an array but we use just one entry.

*/


#define ASY_NBCTX             1        /* Number of used USRT                */
#define BUF_SIZE             64        /* Circular buffer size               */
#define NUM_OF_SEG            2        /* Number of segments of circular buff*/
#include <drv_asy.h>                   /* asy_nbiod asy_nbvl asy_state       */

#define ASY_STM32_NBCTX       1        /* Number of used STM32 UARTs         */
#include <asy_stm32.h>                 /* asy_ctx_stm32[]                    */


static const Asy_def_stm32             /*                                    */
               asy_devdef[ASY_NBCTX] = /* Low level device definition        */
  BEGIN                                /* ---------------------------------- */

    2                                , /*** USART2 (default one for ST-LINK) */
    "ASY1_TIM"                       , /* Timer number = 9                   */
    "NO_DMA"                         , /* DMA TX channel name       FF FF FF */
    "NO_DMA"                         , /* DMA RX channel name       FF FF FF */
                                       /* Description des GPIO des signaux   */
                                       /* port, pin, logique; inutilise = -1 */
    HNULL          , 0               , /* CTS                                */
    HNULL          , 0               , /* DSR                                */
    HNULL          , 0               , /* RI                                 */
    HNULL          , 0               , /* DCD                                */
    HNULL          , 0               , /* DTR                                */
    HNULL          , 0               , /* RTS                                */
    "ASY1_RX"      , 0               , /* RXB     USART2 RW                  */
    "ASY1_TX"      , 0               , /* TXB     USART2 TX                  */

  END                                ; /* ---------------------------------- */

static const
    Asy_drvdef asy_drvdef[ASY_NBCTX] = /* Device definition                  */
  BEGIN                                /* ---------------------------------- */

    0                                , /* Index du device pour le bas-niveau */
    BR_ALL & ~((1 << BR_50) |          /* Vitesses supportees : vitesses >   */
      (1 << BR_75)  | (1 << BR_110) |  /* 4000000 / 16 / 2^12 / 0,2 (+/-10%  */
      (1 << BR_134) | (1 << BR_150) |  /* d'erreur max) # 305 bauds          */
      (1 << BR_200))                 , /*                                    */
    ST_2 | ST_1                      , /* Only 1 or 2 stop bits              */
    0                                , /* Mode 5 bits non supporte           */
    0                                , /* Mode 6 bits non supporte           */
    PAR_ODD | PAR_EVEN               , /* Mode 7 bits avec parite uniquement */
    PAR_ALL                          , /* Mode 8 bits complet                */
    PAR_NONE                         , /* Mode 9 bits sans parite uniquement */
    0x00                             , /* Rien n'est supporte                */
                                       /*                                    */
  END                                ; /* ---------------------------------- */

static const Dev_link asy_dev_link[] = /* USART2 USART3                      */
  BEGIN                                /* ---------------------------------- */
    "ASY USART2"                     , /* dev_name System device name        */
    &asy_desc_stm32                  , /* dev_desc Low-level procedure addr  */
    &asy_devdef[0]                   , /* dev_def  Low-level parameters addr */
    &asy_drvdef[0]                   , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link asy_sdrv_link=  /* ASY system driver                  */
  BEGIN                                /* ---------------------------------- */
    &asy_sdrv_desc                   , /* sdrv_desc  Driver interface        */
    asy_dev_link                     , /* dev_link   List of devices         */
    ASY_NBCTX                        , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.2  VMIO character drivers configuration data
    ==============================================

    9.2.1  ASY driver
    -----------------

    The ASY driver handles Universal Asynchronous Receiver Transmitter devices 
(UART). Despite the fact that some ST devices are USART able (Universal
Asynchronous and Synchronous Receiver Transceiver), the ASY driver does not
handles synchronous modes. One has to note that the "drv_asy.c" module handles
TWO distinct drivers : A system driver (see 9.1.12 ASY driver - asy_sdrv_link)
and a the present character driver. The character driver needs the following
configuration data, in top of the system driver configuration data:

    - One "Drv_link" structure, here "asy_cdrv_link", which is pointed to by
      the "drv_l152" VMIO driver table. So, the "drv_l152" has two entries
      corresponding to "drv_asy.c". One one side, "drv_l152" is pointing to 
      "asy_sdrv_link", the system ASY driver. On the other side, to the second
      structure "asy_cdrv_link". The "drv_desc" field is pointing to 
      "asy_drv_desc", which is located in "drv_asy.c". The "dev_link" field is
      pointing to an array of 3 "Dev_link" structures, here "cdev_asy[3]".

    - The "cdev_asy[]" array contents are the same ones as "asy_dev_link[]",
      the array of "Dev_link" for the system driver. Only the 3 devices names
      are different. One have to note that the system driver is ALWAYS 
      initialized before the character driver and that an UART device which has
      been initialized by the system driver does not need to be initialized
      again by the character driver. So, the "cdev_asy" is useless. However, if 
      a device is removed from "asy_dev_link" (system driver) then it MUST be 
      present in "cdev_asy[]". This is the reason why we fill "cdev_asy", even
      if its data may be of no use.
*/

static const Dev_link  cdev_asy[] =    /* Device ASY                         */
  BEGIN                                /* ---------------------------------- */
    "ASY1"                           , /* dev_name character device name     */
    &asy_desc_stm32                  , /* dev_desc Low-level procedure addr  */
    &asy_devdef[0]                   , /* dev_def  Low-level parameters addr */
    &asy_drvdef[0]                   , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Drv_link asy_cdrv_link =  /* Driver ASY                         */
  BEGIN                                /* ---------------------------------- */
    &asy_drv_desc                    , /* drv_desc   Driver interface        */
    cdev_asy                         , /* dev_link   List of devices         */
    ASY_NBCTX                        , /* nb_dev     Count of devices        */
    HNULL                              /* pipe       Pipe interface address  */
  END                                ; /* ---------------------------------- */


/*  9.2.2 GPIO driver

    As the ASY driver, the GPIO driver has two software interfaces, its
system driver interface (see "9.1.10 GPIO driver - gpio_sdrv_link") and the
present character driver interface. The GPIO character interface only needs
one "Drv_link" structure, here "gpio_cdrv_link", which is pointed to by the
"drv_l152" VMIO list of driver. So, "drv_l152" points to "gpio_sdrv_link" and
also to "gpio_cdrv_link".

    The "drv_desc" field of "gpio_cdrv_link" points to the "gpio_drv_desc"
structure from "drv_gpio.c". The "dev_link" field points to the "gpio_dev_link"
structure which can be found at "9.1.10 GPIO driver - gpio_sdrv_link". In fact,
initialization of the unique GPIO controller we use here is done by the GPIO
system driver.

    Besides this configuration data, the "drv_gpio.c" needs a const table which
name must be "gpio_tsubcnt". This table indicates the number of GPIO pins for
each device, for each controller, in our case we have only one controller, which
managed all the GPIO_NBPIN pins.

*/

static const Drv_link gpio_cdrv_link = /* GPIO character driver              */
  BEGIN                                /* ---------------------------------- */
    &gpio_drv_desc                  ,  /* drv_desc   Driver interface        */
    gpio_dev_link                   ,  /* dev_link   List of devices         */
    GPIO_NBIOD                      ,  /* nb_dev     Count of devices        */
    HNULL                              /* pipe       Pipe interface address  */
  END                                ; /* ---------------------------------- */

/* Needed by "drv_gpio.c" ...................................................*/

USHORT gpio_tsubcnt [GPIO_NBIOD] = { GPIO_NBPIN } ;

/*
    9.3  Block drivers definitions

    9.4  Volume drivers definitions

    9.5 Protocols definitions

    9.6 Services definitions

    9.7  VMIO driver list and other global data structures

          9.7.1  VMIO driver list - drv_l152
          9.7.2  Board_link structure - board_l152
          9.7.3  List of Drv_itf contexts - drv_req_ctx


    9.7.1  VMIO driver list - drv_l152
*/

static const Drv_link* const drv_l152[]= /* VMIO list of drivers             */
  BEGIN                                  /* -------------------------------- */

/* System drivers .......................................................... */

    (const Drv_link *)&sys_sdrv_link    , /*  0 SYSTEM                       */
    (const Drv_link *)&board_sdrv_link  , /*  1 BOARD                        */
    (const Drv_link *)&ramh_sdrv_link   , /*  2 RAM_HARD                     */
    (const Drv_link *)&rams_sdrv_link   , /*  3 RAM_SOFT                     */
    (const Drv_link *)&vmit_sdrv_link   , /*  4 VMIT                         */
    (const Drv_link *)&tim_sdrv_link    , /*  5 TIME                         */
    (const Drv_link *)&vmio_sdrv_link   , /*  6 VMIO                         */
    (const Drv_link *)&gpio_sdrv_link   , /*  7 GPIO                         */
    (const Drv_link *)&dma_sdrv_link    , /*  8 DMA                          */
    (const Drv_link *)&i2c_sdrv_link    , /*  9 I2C                          */
    (const Drv_link *)&asy_sdrv_link    , /* 10 ASY                          */
    (const Drv_link *)&rslv_sdrv_link   , /* 11 RESOLVE (avant carac)        */

/* Characters drivers ...................................................... */

    (const Drv_link *)&asy_cdrv_link    , /* 12 ASY                          */
    (const Drv_link *)&gpio_cdrv_link   , /* 13 GPIO                         */

/* Block drivers ........................................................... */

/* Volume drivers .......................................................... */

/* Protocols ............................................................... */

/* Services ................................................................ */

/* HNULL end of list marker ................................................ */

    HNULL                            , /* END OF LIST marker                 */
  END                                ; /* ---------------------------------- */


/*  9.7.2  Board_link structure - board_l152
*/

Board_link board_l152 =                /* Board Discovery kit Nucleo-L152RE  */
  BEGIN                                /* ---------------------------------- */
    "STM32L152RE"                    , /* Nom du board                       */
    drv_l152                         , /* Liste des drivers                  */
  END                                ; /* ---------------------------------- */

/*  9.7.3  List of Drv_itf contexts - drv_req_ctx

    WARNING: This table uses as an index the major number, which is intended
    to be used as the "vmos_driver[]" index ONLY. So one has to take care to
    always simultaneoulsy update both tables accordingly.
*/

Drv_req_ctx drv_req_ctx[] =
  BEGIN
/*   itfctx          sub_lw        nbiod        nbmodd   sessid_cond_act[]   */
/*   -------------   -----------   ----------   ------   -----------------   */
  {  HNULL         , HNULL       , 0          , 0      , {0}              } ,
  {  HNULL         , HNULL       , 0          , 0      , {0}              } ,

  {  asy_itfctx    , HNULL       , ASY_NBCTX  , 1      , {0}              } ,
  {  gpio_itfctx   , gpio_sub_lw , GPIO_NBCTL , 1      , {0}              } ,

  END                                ;


/*
    10   Beginning of the code
    __________________________

board_init       Board dependant initialization procedure
vm_sleep         Enter CPU sleep mode, here WFI
init_aut_script  Empty procedure, called by "vmk1.c"
init_aut_panel   Empty procedure, called by "vmk1.c"
sdrv_empty       Empty procedure for system drivers init/end
*/

/*  Procedure board_init ----------------------------------------------------*/
/*
    Purpose : Initialization procedure of the BOARD system driver. In fact,
              the purpose of the BOARD system driver is to allow "init_hypsys"
              to call "board_init" and nothing more. We remind that "board_init"
              is the second procedure to be called by "init_hypsys", just after
              the initialization procedure of the SYSTEM driver.

    Author : JYA                                 Last modification : 02.04/2022

...............................................................................

    Calling example : board_init(drv)

    Input     : drv      IWORD      Index in the VMIO driver table

    Output    : valproc  IWORD      Always 0

    Notes : 

...............................................................................
*/

static IWORD  PROCEDURE board_init(IWORD drv)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          ldev              ; /* SET timer number 1..14             */
    IWORD          chan              ; /* Optional TIM channel number        */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("board_init")            ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Activate the use of CPU sleep, whenever "vm_sleep" is called    *
 * -------   We simply call "set_idle(1)"                                    *
 *           --------------------------------------------------------------- *
 *           VERY IMPORTANT NOTICE : When using GDB/OpenOCD, we have a       *
 *           regular polling of the target by the debugger or OpenOCD. But   *
 *           when CPU is put in IDLE, it seems it does not answers any more  *
 *           to GDB/OpenOCD requests and we get on the GDB window:           *
 *           DAP transaction stalled (WAIT) - slowing down                   *
 *           and then using GDB becomes very difficult or impossible. What   *
 *           is sure, we never get this message if "set_idle(1)" is not      *
 *           called. Therefore, we have a "use_gdb" global variable which    *
 *           is set by "gdb.cfg" to 0x12345678 and we call "set_idle(0)" if  *
 *           variable is 0x12345678 (do not use IDLE mode) else with any     *
 *           other value we call "set_idle(1)"                               *
 *           --------------------------------------------------------------- *
 *           The "use_gdb" address is defined by "linker.cfg" as a linker    *
 *           symbol, so an address. If we want the get the value which is    *
 *           located at tha adress, we have to use "*use_gdb"                *
 *****************************************************************************/

                                       /* If GDB has written at address      */
   set_idle(use_gdb EQ 0x12345678  ?   /* "used_gdb" the 0x12345678 value    */
            0                      :   /* then we call "set_idle(0)" else    */
            1                      ) ; /* we call "set_idle(1)"              */

/*****************************************************************************
 * Step  2 : Initialize global variable holding timer numbers :              *
 * --------  - The "init_timarmv7m" procedure form "tim_arm.c" needs the     *
 *             "sys_tim" global variable as input, it should hold the ST     *
 *             defined TIM number we use a the system "tick". Here, we scan  *
 *             our "ref_tim32[]" table, looking for the "SYS_TIM" name, and  *
 *             we get "sys_tim"                                              *
 *           - The "filter_stm32" from "tim_stm32.c" needs a timer and       *
 *             assumes that the "flt_tim" global variable holds the ST TIM   *
 *             number. In our case, the FLT_TIM entry of "ref_tim32[]" holds *
 *             that value and we use the "get_ref_tim_stm32" procedure in    *
 *             order to retreive it.                                         *
 *****************************************************************************/

    get_ref_tim_stm32("SYS_TIM"  ,    /* Timer name for this configuration   */
                       &ldev     ,    /* Corresponding ST TIM number 1..14   */
                       &chan      ) ; /* Optional channel number (not used)  */
                                      /*                                     */
    sys_tim = ldev                  ; /* Keep in the "sys_tim" gloaal var    */
                                      /*                                     */
    get_ref_tim_stm32("FLT_TIM"  ,    /* Timer name for this configuration   */
                       &ldev     ,    /* Corresponding ST TIM number 1..14   */
                       &chan      ) ; /* Optional channel number (not used)  */
                                      /*                                     */
    flt_tim = ldev                  ; /* Keep in the "sys_tim" gloaal var    */


/*****************************************************************************
 * Step  3 : Start the DWT (Data Watchpoint and Trace Unit) CPU clock        *
 * -------   pulses counter. We have to set bit 0 of the DWT_CTRL register   *
 *           (see page 731 and 737 of ARM DDI0403E.e). Addresses of the DWT  *
 *           registers are given on page 9-4 of ARM DDI0439B, we get:        *
 *           - 0xE0001000 DWT_CTRL      Control register                     *
 *           - 0xE0001004 DWT_CYCCNT    Cycle Count Register                 *
 *           - 0xE0001010 DWT_SLEEPCNT  Sleep Count register                 *
 *           - 0xE000EDFC DEMCR
 *           On page C1-741 of ARM DDI 0403E.e we can read "CYCCNT increments*
 *           on each processor cycle cycle when DWT_CTRL.CYCNTENA == 1 and   *
 *           DEMCR.TRCENA == 1. On page C1-706 of ARM DDI 0403E.e we see     *
 *           that bit TRCENA is b24 of DEMCR. We also read that if bit is 1  *
 *           then DWT and ITM units are enabled                              *
 *****************************************************************************/

    *(UINT*)0xE0001000 |= 1          ; /* Set bit b0 of DWT_CTRL             */
    *(UINT*)0xE000EDFC |= 1 << 24    ; /* DWT and ITM units enabled          */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("board_init")            ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */

/*  Procedure vm_sleep ------------------------------------------------------*/
/*
    Purpose : Allow the CPU to enter it sleep mode, if any

    Author : JYA                                 Last modification : 01/13/2022

...............................................................................

    Calling example : vm_sleep( par )

    Input     : par      IWORD      0:Do not enter sleep mode 1:Enter sleep

    Output    : valproc  IWORD      Always 0

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE vm_sleep(IWORD par)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("vm_sleep")              ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : If "par" is set, we execute the Wait For Irq ARM machine        *
 * -------   instruction: The CPU enters low power mode until and IRQ is     *
 *           raised.                                                         *
 *****************************************************************************/

    GPIO_CPU(0)                      ; /* Low-level = inactive               */
                                       /*                                    */
    if (par) asm("WFI")              ; /* Si par = 1, on utilise WFI.        */
                                       /*                                    */
    GPIO_CPU(1)                      ; /* High-level = active                */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("vm_sleep")              ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */

IWORD PROCTEST init_aut_script(UINT *id)
  BEGIN
    *id = 0xFFFFFFFF                  ;
    return ( 0 )                      ;
  END_PROC

IWORD PROCTEST init_aut_panel(UINT *id)
  BEGIN
    *id = 0xFFFFFFFF                  ;
    return ( 0 )                      ;
  END_PROC

IWORD PROCEDURE sdrv_empty(IWORD drv) 
  BEGIN
    return ( 0 )                      ;
  END_PROC
