/*****************************************************************************/
/*                                                                           */
/*  HyperPanel OS                       File "src/space_appcont_stm32m4.c"   */
/*  Board - Configurable static data for stm32m4.                            */
/*                                                                           */
/*  Author : JYA/TE                     Last modification : 09/01/2023       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


    1    Purpose of this module
    ___________________________


         The module "space_appcont.c" contains all the static RAM data of the
depends on the target. The present file may hold defines followed by include
files, such as "vmk.h", which olds the arrays that are using the define.


    2    User manual
    ________________


    2.1  Contents of the present file

    3    Porting guide
    __________________


    4    Include files and external references
    __________________________________________
*/

#include <stdtyp.l>                    /* Hyperpanel definitions & constants */
#include <event.l>                     /* Needed by drv_req.h                */
                                       /*                                    */
#include <aut.h>                       /* S_trans                            */
#include <vmk.h>                       /* Task_grp                           */
#include <drv.h>                       /* Sys_driver                         */
#include <drv_req.h>                   /* Drv_req_ctx                        */

/*
    5    Inner workings of the module
    _________________________________


    6    Exported (public) variables
    ________________________________

         6.1  Group of tasks - const task_grp[]
         6.2  VMK list of drivers - const vmos_driver[] & drv_sys_evt_hook[]
         6.3  VMK RAM Tables and stacks
         6.4  Memory allocation tables
         6.5  "tsk_iolib.c" tables
         6.6  Misceleanous
*/


#define SPACE_APPCONT_XYZ_DATA_C

/*  6.1  Group of tasks - const task_grp[]
    --------------------------------------

    This const table is needed by the "init_vmk" procedure (vmk1.c). The
    "init_vmk" does a loop on the table entries and the loops stops with
    the first entry with a "use" field is greater than 2.

    The "autostart" functions are here defined as "weak" symbols, thus enabling
    the application de redefine those. The "task_grp" table thus may be
    re-used by different applications that will ne need to define empty
    functions.
*/

int __attribute__((weak)) init_app_aut(Task_grp *g,char* mod,char*conf) 
  { return 0; }

int __attribute__((weak)) init_app_tsk(Task_grp *g,char* mod,char*conf) 
  { return 0; }

const Task_grp task_grp[] =            /* Definition des groupes de taches   */
  BEGIN                                /* ---------------------------------- */
                                       /* (group use autostart procstart)    */
/*  group      use autostart procstart    ---------------------------------- */
                                       /* Groupes de taches "temps reel" ... */
    "appaut" , 1 , 1 , init_app_aut ,  /* VMK User FSM start                 */
    "apptsk" , 1 , 1 , init_app_tsk ,  /* VMK User tasks start               */
                                       /* ---------------------------------- */
                                       /* Groupes de taches "standard" ..... */
                                       /* -                                  */
                                       /* ---------------------------------- */
    ""     , 9 , 9 , HNULL           , /* End of list mark ................. */
  END                                ; /* ---------------------------------- */


/*  6.2  VMK list of drivers - const vmos_driver[] & drv_sys_evt_hook[]
    -------------------------------------------------------------------

    This const table is needed by the "vmk" procedure (vmk1.c). The "vmk"
    procedure is called from the Application container main loop for each
    external event (comming from the IO container. The first step of "vmk" is
    to call an event procedure: vmos_driver[major]->drv_evt(major,evt) where
    "major" is the vvalue of bits b16..23 of the "reserve" field of the incoming
    event.

    Entry 0 of "vmos_driver[]" MUST be "sysevt_driver", in other words the
    major number for "sysevt_driver" MUST be 0. This is because by convention
    all the system drivers of the IO containter are using a "dummy" IOD value
    that has the form 0xFF00ddss", so with a 00 major number.

    The "drv_sys_evt_hook[]" const table is holding C functions addresses, we
    have one function for each system event code (EVT_I2C_END, EVT_SPI_END)
*/

extern const Sys_driver sysevt_driver; /*                                    */
extern const Sys_driver membuf_driver; /*                                    */
extern const Dps_driver asy_driver   ; /*                                    */
extern const Dps_driver gpio_driver  ; /*                                    */
extern const Dps_driver arp_driver   ; /*                                    */
extern const Dps_driver ip_driver    ; /*                                    */
extern const Dps_driver icmp_driver  ; /*                                    */
extern const Dps_driver udp_driver   ; /*                                    */
extern const Dps_driver tcp_driver   ; /*                                    */
extern const Dps_driver dns_driver   ; /*                                    */

const Sys_driver* const vmos_driver[]= /* Drivers table for stm32m4.         */
  BEGIN                                /* System drivers (no tags) --------- */
    (Sys_driver*)&sysevt_driver      , /* 0 MUST be entry 0                  */
    (Sys_driver*)&membuf_driver      , /* 1                                  */
                                       /* Character drivers (tags) --------- */
    (Sys_driver*)&asy_driver         , /* 2                                  */
    (Sys_driver*)&gpio_driver        , /* 3                                  */
    HNULL                            , /* 4                                  */
  END                                ; /*                                    */

const USHORT vmos_nbdriver = 4       ; /* Count of table entries             */

const IWORD (*drv_sys_evt_hook[])()=
  BEGIN
    HNULL                            , /* DRV_IO_END       400               */
    HNULL                            , /* DRV_ASY_END      401               */
    HNULL                            , /* DRV_I2C_END      402               */
    HNULL                            , /* DRV_SPI_END      403               */
    HNULL                            , /* DRV_USB_END      404               */
    HNULL                            , /* DRV_ETH_END      405               */
    HNULL                            , /* DRV_RAD_END      406               */
  END                                ;

/*  6.3  VMK RAM Tables and stacks
    ------------------------------

    6.3.1  Sizes of "vmk.c" tables
    ------------------------------
*/

#define VMK_AUT_MAX          35        /* Max number of FSM's                */
#define VMK_IQ_EVT_MAX        8        /* Size of "fai_evt[]"                */
#define VMK_ROUTE_MAX         8        /* Max number of event's route        */
#define VMK_SESS_COND_MAX     2        /* Max number of conditions           */
#define VMK_TO_MAX           16        /* Max number of Time-out's           */
#define VMK_TASK_MAX          5        /* Max number of tasks                */
#define VMK_SEM_MAX           2        /* Max number of semaphores           */
#define VMK_OWNER_MAX        10        /* Max number of owner                */


/*  6.3.2  Counts and sizes of static task stacks
    -------------------------------------------
*/

#define NB_STACK_SZ           1        /* Count of stack sizes               */

#define STACK_SZ0             1        /* Stack size in KB                   */
#define STACK_SZ1             0        /* Stack size in KB                   */
#define STACK_SZ2             0        /* Stack size in KB                   */
#define STACK_SZ3             0        /* Stack size in KB                   */
#define STACK_SZ4             0        /* Stack size in KB                   */

#define STACK_NB0             1        /* Count of stacks of size SZ0        */
#define STACK_NB1             0        /* Count of stacks of size SZ1        */
#define STACK_NB2             0        /* Count of stacks of size SZ2        */
#define STACK_NB3             0        /* Count of stacks of size SZ3        */
#define STACK_NB4             0        /* Count of stacks of size SZ4        */

/*  6.3.3  Include the tables definitions
    -------------------------------------
*/
#include <vmk.h>

/*  6.4  Memory allocation tables
    -----------------------------
*/

#define MMAXPART              1        /* Size of "mem_ctx[]"                */
#include <mem_user.h>

/*  6.5  tsk_iolib.c tables
    -----------------------
*/
#define NB_MODD               5        /* Count of modules and MODD          */
#define NB_IOD               20        /* Count of IOD's                     */

#include <tsk_iolib.h>

const Mod_def iolib_moddef[NB_MODD] =
  BEGIN
/*  modname  flags         nbdev  rtemincode      rtemaxcode      */
/*  -------  ------------  -----  -------------   --------------- */
    "ASY"  , NBLOCKING_IO ,  1  , IND_REPORT    , IND_REPORT      ,
    "GPIO" , 0            ,  1  , IND_REPORT    , IND_REPORT      ,
    "LED"  , 0            ,  1  , -1            , -1              ,
    "KBD"  , 0            ,  1  , DRV_KEY_PRESS , DRV_KEY_RELEASE ,
    "CAN"  , NBLOCKING_IO ,  1  , IND_REPORT    , IND_REPORT      ,
  END                         ;

/*  6.6   Misceleanous
    ------------------
*/
    UINT            hmsg_flag        ; /* Masque des messages de trace.      */
    IWORD           vlgv             ; /*                                    */

   void PROCEDURE kbd_it_press  (INT dev, INT keyboard, INT param) {}
   void PROCEDURE kbd_it_release(INT dev, INT keyboard           ) {}


