/*****************************************************************************/
/*                                                                           */
/*  HyperPanel OS                       File "stm32m4/src/vm.h"              */
/*  Board - Constants and definitions specific to stm32m4.                   */
/*                                                                           */
/*  Author : JYA                        Last modification : 07/11/2023       */
/*                                                                           */
/*****************************************************************************/


#ifndef VM_H                           /* Begin conditionnal compilation     */
#define VM_H                           /* Identifier for this include        */

/* General defines and include files ........................................*/

#define BOARD_STM32M3                  /* Board identifier                   */

#define _LITTLE_ENDIAN_                /* CPU words byte order is Intel      */
#define HYP_BYTE_ORDER   1             /* CPU words byte order is Intel      */

#include <stdarg.h>                    /* Variable lenth argument lists      */


/* Elementary data types needed by "stdtyp.l" ...............................*/

typedef char               HYPER_CHAR  ; /* Signed integer          1 byte   */
typedef unsigned char      HYPER_UCHAR ; /* Unsigned integer        1 byte   */
typedef short              HYPER_SHORT ; /* Signed integer          2 bytes  */
typedef unsigned short     HYPER_USHORT; /* Unsigned integer        2 bytes  */
typedef int                HYPER_INT   ; /* Signed integer          4 bytes  */
typedef unsigned int       HYPER_UINT  ; /* Entier non signe        4 bytes  */
typedef long long          HYPER_LONG  ; /* Signed integer          8 bytes  */
typedef unsigned long long HYPER_ULONG ; /* Unsigned integer        8 bytes  */
typedef float              HYPER_FLOAT ; /* Float                   4 bytes  */
typedef double             HYPER_DOUBLE; /* Float                   8 bytes  */


/* Remarkable elementary data types values ..................................*/

#define HYPER_MAXLONG  (int)0x7FFFFFFF /* Maximum value for LONG/HYPER_LONG  */
#define HYPER_MAXULONG (int)0xFFFFFFFF /* Maximum value for ULONG/HYPER_ULONG*/

#define HNULL          0               /* NULL pointer/address               */

#define H0             0               /*  0 as a 4 bytes value              */
#define H1             1               /*  1 as a 4 bytes value              */
#define HN            -1               /* -1 as a 4 bytes value              */
#define H2             2               /*  2 as a 4 bytes value              */


/* Local/global standard sizes ..............................................*/

#define STDEXT         (INT)4          /* File name extension size in bytes  */
#define STDNAME        (INT)32         /* File name length in bytes          */
#define MAX_DFNAME     (INT)8          /* Disk file name, not including 0    */


/* Variable length argument lists ...........................................*/

#define VA_ALIST       CHAR *a,...     /*                                    */
#define VA_DCL                         /*                                    */
#define VA_LIST        va_list         /*                                    */
#define VA_START(x)    va_start(x,a)   /*                                    */
#define VA_1ST(x,y)    a               /*                                    */
#define VA_ARG(x,y)    va_arg(x,y)     /*                                    */
#define VA_END(x)      va_end(x)       /*                                    */


/* MOTOROLA integer byte order to CPU byte order ............................*/

/* Written like this because these macros are converted into a single ARM    */
/* assembly instruction.                                                     */

#define SWAP2(a)                       /* Converts a 2 bytes MOTOROLA        */\
 do { unsigned short *v = a          ; /* integer to a 2 bytes CPU integer.  */\
    *v = (*v >> 8)                     /*                                    */\
       | (*v << 8)                   ; /*                                    */\
 } while (0)                           /*                                    */

#define SWAP4(a)                       /* Converts a 4 bytes MOTOROLA        */\
 do { unsigned int *v = a            ; /* integer to a 4 bytes CPU integer.  */\
    *v = ((*v >> 24)             )     /*                                    */\
       | ((*v >>  8) & 0x0000FF00)     /*                                    */\
       | ((*v <<  8) & 0x00FF0000)     /*                                    */\
       | ((*v << 24)             )   ; /*                                    */\
 } while (0)                           /*                                    */

#define SWAPN(a,n)                     /* Converts a "n" bytes MOTOROLA      */\
 do {int _c,_j;UCHAR *_a=(UCHAR*)(a);  /* integer to a "b" bytes CPU integer.*/\
    for(_j=0;_j LT ((n)>>1);_j++)                                              \
    {_c=*(_a+_j);*(_a+_j)=*(_a+(n)-1-_j);*(_a+(n)-1-_j)=_c;}} while(0)


/* CPU integer byte order to MOTOROAL byte order ............................*/
 
#define MOTOROLA_2(a)   SWAP2(a)       /* Converts a 2 bytes CPU integer     */
                                       /* to a 2 bytes MOTOROLA integer.     */

#define MOTOROLA_4(a)   SWAP4(a)       /* Converts a 4 bytes CPU integer     */
                                       /* to a 4 bytes MOTOROLA integer.     */

#define MOTOROLA_N(a,n) SWAPN(a,n)     /* Converts a "n" bytes CPU integer   */
                                       /* to a MOTOROLA integer.             */


/* CPU integer byte order to INTEL byte order ...............................*/

#define INTEL_2(a)                     /* Do not change anything.            */
#define INTEL_4(a)                     /* Do not change anything.            */
#define INTEL_8(a,n)                   /* Do not change anything.            */


/* Translations between device address and CPU address ......................*/

/* . PHYS_AD transforms a CPU address into a physical address, for direct    */
/*   use by a device. This macro is voluntarily not caste as a pointer,      */
/*   because from the CPU point of view this address is not valid.           */

#define PHYS_AD(x) (UINT)(x)

/* . RAM_AD transforms a physical address used by a device into an address   */
/*   out of the box address usable by the CPU through an MMU.                */

#define RAM_AD(x)  (UCHAR*)(x)

/* . CPU_AD transforms a physical used by a device into an address in the    */
/*   box usable by the CPU through an MMU.                                   */

#define CPU_AD(x)  (UCHAR*)(x)


/* Memory barrier ...........................................................*/

/* A memory barrier is an instruction that makes the CPU wait for the end of */
/* all previous writes to RAM. This instruction is typically used after      */
/* writing data to RAM to be read by the hardware, just before asking the    */
/* hardware to read the data.                                                */
/*                                                                           */
/* A Data Memory Barrier is used. ARMv7-M processors also offer a Data       */
/* Synchronization Barrier. The first one forbids any transaction with the   */
/* memory located after the barrier in the order of the program to be        */
/* executed as long as the transactions with the memory which precedes it    */
/* are not finished. The second one is stronger and forbids the execution of */
/* ANY instruction after the barrier.                                        */

#define FLUSH_VAR(var) BEGIN __asm__ __volatile__ ("DMB"); END


/* Memory alignment .........................................................*/

/* The most recent C standard (C11 and after) specifies an oparetor for      */
/* alignment purposes. It is not the same one, but it is recognized by GCC   */
/* other C compilers. The most portable until proven otherwise is to put it  */
/* between the type and the name of the first field:                         */
/*                                                                           */
/*     typedef struct                             /+ S                   +/  */
/*       BEGIN                                    /+ ------------------- +/  */
/*         int ALIGN(16) a                      ; /+ Champ a             +/  */
/*         int b                                ; /+ Champ b             +/  */
/*         int c                                ; /+ Champ c             +/  */
/*       END S                                  ; /+ ------------------- +/  */

#define ALIGN(x) __attribute__((aligned(x)))


/* Memory rewritten library procedures prototypes ...........................*/

void  *memset   ( void *       , int          , unsigned int );
void  *memcpy   ( void *       , const void * , unsigned int );
void  *memmove  ( void *       , const void * , unsigned int );
int    memcmp   ( const void * , const void * , unsigned int );


/* Text rewritten library procedures prototypes .............................*/

int    strcmp   ( const char * , const char *                );
int    strncmp  ( const char * , const char * , unsigned int );
char  *strcpy   ( char *       , const char *                );
char  *strncpy  ( char *       , const char * , unsigned int );
char  *strcat   ( char *       , const char *                );
char  *strncat  ( char *       , const char * , unsigned int );
char  *strchr   ( const char * , int                         );
char  *strrchr  ( const char * , int                         );
unsigned int strlen ( const char *                           );


#endif                                 /* End of conditionnal compilation.   */
