/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/board.h"               */
/*  Constants and definitions for board module.                              */
/*                                                                           */
/*  Author : JYA/FA                     Last modification : 06/25/2023       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Devices use attribute values

         2  Structures and typedefs
            2.1  System driver interface (ROM) - Sdrv_desc
            2.2  Device interface (ROM) - Dev_link
            2.3  System driver description (ROM) - Sdrv_link
            2.4  Telecom/pipe interface (ROM) - Tlcm_desc
            2.5  Driver/protocol/service interface (ROM) - Drv_desc
            2.6  Driver pipe support (ROM) - Pipe_desc
            2.7  Character driver description (ROM) - Drv_link
            2.8  Board description (ROM) - Board_link

         3  Public variables
            3.1  Board description
            3.2  System driver interface
            3.3  Character driver interface
            3.4  Block driver interface

         4  Prototypes of public procedures

*/


#ifndef BOARD_H                        /* If this file is not already        */
#define BOARD_H                        /* included, include it.              */


/*
    1    Defines
    ____________

    1.1  Devices use attribute values
    ---------------------------------
*/

#define UUSR             0             /* User code may use the hardware     */
#define USYS             1             /* System code may use the hardware   */
#define UHRT             2             /* Hard Real-Time code may use it     */
/*
    2    Structures and typedefs
    ____________________________


    2.1  System driver interface (ROM) - Sdrv_desc
    ----------------------------------------------
*/

typedef struct                         /* VMIO system driver interface       */
  BEGIN                                /* ---------------------------------- */
    const CHAR  *drv_name            ; /* Driver name                        */
    UCHAR       typdrv               ; /* Always 0 with a system driver      */
                                       /* Interface procedures ------------- */
    IWORD       (*init)()            ; /* Initialization procedure           */
    IWORD       (*end )()            ; /* Termination procedure              */
  END           Sdrv_desc            ; /* ---------------------------------- */


/*
    2.2  Device interface (ROM) - Dev_link
    --------------------------------------
*/

typedef struct                         /* Device interface                   */
  BEGIN                                /* ---------------------------------- */
    const CHAR    *dev_name          ; /* System device name (unused)        */
    const void    *dev_desc          ; /* Low-level procedure addresses      */
    const void    *dev_def           ; /* Low-level parameters addresses     */
    const void    *drv_def           ; /* High-level parameters addresses    */
  END             Dev_link           ; /* ---------------------------------- */
 

/*
    2.3  System driver description (ROM) - Sdrv_link
    ------------------------------------------------
*/

typedef struct                         /* System driver description          */
  BEGIN                                /* ---------------------------------- */
    const Sdrv_desc *sdrv_desc       ; /* Driver interfaces address          */
    const Dev_link  *dev_link        ; /* Device interfaces address          */
    UCHAR         nb_dev             ; /* Number of devices in sdev_link[]   */
  END             Sdrv_link          ; /* ---------------------------------- */
 

/*
    2.4  Telecom/pipe interface (ROM) - Tlcm_desc
    ---------------------------------------------

    Functions specifications are at "3.2  Layer interface specification" of
    "req_lib.c".
*/

typedef struct                         /* VMIO telecom/pipe interface        */
  BEGIN                                /* ---------------------------------- */
    IWORD         level              ; /* 1=driver, 2 and + = protocol       */
    CHAR          *name              ; /* layer name                         */
                                       /* Get Context/info addr ------------ */
    IWORD         (*get_rctx)   ()   ; /* Returns context address Req_ctx.   */
    IWORD         (*get_info)   ()   ; /* Returns requested value address.   */
                                       /* Interface procedure addr --------- */
    IWORD         (*alloc_sub)  ()   ; /* Unused                             */
    IWORD         (*free_sub)   ()   ; /* Unused                             */
    IWORD         (*connect)    ()   ; /* Request for output connection.     */
    IWORD         (*listen)     ()   ; /* Waiting for input connection.      */
    IWORD         (*disconnect) ()   ; /* Disconnect request.                */
    IWORD         (*abort)      ()   ; /* Aborts an ongoing connection       */
    IWORD         (*start)      ()   ; /* Start flow request.                */
    IWORD         (*stop)       ()   ; /* Stop flow request.                 */
    IWORD         (*send)       ()   ; /* Send request.                      */
    IWORD         (*receive)    ()   ; /* Receive request.                   */
    IWORD         (*signal)     ()   ; /* Send a signal to remote entity     */
    IWORD         (*setval)     ()   ; /* Write register request.            */
    IWORD         (*getval)     ()   ; /* Read register request.             */
    IWORD         (*rpc)        ()   ; /* Command execution request.         */
    IWORD         (*io)         ()   ; /* Input/Output request.              */
  END             Tlcm_desc          ; /* ---------------------------------- */

/*
    2.5  Driver/protocol/service interface (ROM) - Drv_desc
    -------------------------------------------------------
*/

typedef struct                         /* VMIO driver/proto/serv interface   */
  BEGIN                                /* ---------------------------------- */
    const CHAR    *drv_name          ; /* Driver name                        */
    UCHAR         typdrv             ; /* 001:dcar 010:proto 011:ser 101:dblk*/
                                       /* Initialization ------------------- */
    UCHAR         *initdone          ; /* Initialization flag address        */
    IWORD         (*init)()          ; /* Specific initialization proc addr  */
                                       /* Automaton parameters ------------- */
    UCHAR         *autnum            ; /* VMIO automaton number              */
    const USHORT  lw0                ; /* REQ_ALLOC/REQ_FREE Logical way     */
                                       /* logical way. (default value 0).    */
                                       /* Const counts --------------------- */
    const USHORT  *nblw              ; /* Size in USHORT's of "state[]"      */
    const USHORT  *nbiod             ; /* Size in USHORTs of sessid/con2lw   */
    const USHORT  *nbmodd            ; /* Count of stacking contexts         */
    const USHORT  *msubcnt           ; /* Max Sub-channel count per device   */
                                       /* VMIO telecom/pipe interface ------ */
    const Tlcm_desc *tlcm            ; /* Tlcm_desc Interface procedures     */
    void          *adr               ; /* User address                       */
  END             Drv_desc           ; /* ---------------------------------- */

/*
    2.6  Driver pipe support (ROM) - Pipe_desc
    ------------------------------------------
*/

typedef struct _pipe_desc              /* Driver pipe support interface      */
  BEGIN                                /* ---------------------------------- */

    /* Interface to implement for the source ................................*/

    void        (*get_info_src)()    ; /*                                    */
    IWORD       (*init_src)()        ; /*                                    */
    void        (*end_src)()         ; /*                                    */
    IWORD       (*input)()           ; /*                                    */
    IWORD       (*seek)()            ; /*                                    */
    void        (*free_src_data)()   ; /*                                    */

    /* Interface to implement for the well ..................................*/

    void        (*get_info_sink)()   ; /*                                    */
    IWORD       (*init_sink)()       ; /*                                    */
    void        (*end_sink)()        ; /*                                    */
    IWORD       (*output)()          ; /*                                    */

    /* Common interface .....................................................*/

    void        (*event)()           ; /*                                    */
  END           Pipe_desc            ; /* ---------------------------------- */


/*
    2.7  Character driver description (ROM) - Drv_link
    --------------------------------------------------
*/

typedef struct                         /* Character driver description       */
  BEGIN                                /* ---------------------------------- */
    const Drv_desc  *drv_desc        ; /* Driver interface address           */
    const Dev_link  *dev_link        ; /* Devices interfaces list addresses  */
    UCHAR           nb_dev           ; /* Number of devices in dev_link[]    */
    const Pipe_desc *pipe            ; /* Pipe interface address             */
  END               Drv_link         ; /* ---------------------------------- */
 
 
/*
    2.8  Board description (ROM) - Board_link
    -----------------------------------------
*/
 
typedef struct                         /* Board description                  */
  BEGIN                                /* ---------------------------------- */
    const CHAR      *board_name      ; /* Board name                         */
    const Drv_link  *const* drv_link ; /* Drivers/proto/serv table address   */
  END               Board_link       ; /* ---------------------------------- */
 

/*
    3    Public variables
    _____________________


    3.1  Board description
    ----------------------
*/

extern const Board_link *board          ; /* Board description               */


/*
    3.2  System driver interface
    ----------------------------
*/

extern const Sdrv_desc vmio_sdrv_desc   ; /* VMIO     system drv interface   */
extern const Sdrv_desc ram_sdrv_desc    ; /* RAM      system drv interface   */
extern const Sdrv_desc asy_sdrv_desc    ; /* ASY      system drv interface   */
extern const Sdrv_desc can_sdrv_desc    ; /* CAN      system drv interface   */
extern const Sdrv_desc dma_sdrv_desc    ; /* DMA      system drv interface   */
extern const Sdrv_desc eep_sdrv_desc    ; /* EEP      system drv interface   */
extern const Sdrv_desc gpio_sdrv_desc   ; /* GPIO     system drv interface   */
extern const Sdrv_desc i2c_sdrv_desc    ; /* I2C      system drv interface   */
extern const Sdrv_desc spi_sdrv_desc    ; /* SPI      system drv interface   */
extern const Sdrv_desc vmit_sdrv_desc   ; /* VMIT     system drv interface   */
extern const Sdrv_desc kbd_sdrv_desc    ; /* KEYBOARD system drv interface   */
extern const Sdrv_desc sdrv_mse         ; /* MOUSE    system drv interface   */
extern const Sdrv_desc sdrv_pcr         ; /* PCR      system drv interface   */
extern const Sdrv_desc sdrv_pipe        ; /* PIPE     system drv interface   */
extern const Sdrv_desc rslv_sdrv_desc   ; /* RESOLVE  system drv interface   */
extern const Sdrv_desc sdrv_usb         ; /* USB      system drv interface   */
extern const Sdrv_desc eth_sdrv_desc    ; /* ETH      system drv interface   */
extern const Sdrv_desc can_sdrv_desc    ; /* CAN      system drv interface   */
extern const Sdrv_desc vos_sdrv_desc    ; /* VOS      system drv interface   */
extern const Sdrv_desc sdrv_gra         ; /* GRA      system drv interface   */
extern const Sdrv_desc rad_sdrv_desc    ; /* RAD      system drv interface   */


/*
    3.3  Character driver interface
    -------------------------------
*/

extern const Drv_desc asy_drv_desc      ; /* ASY      char drv interface     */
extern const Drv_desc can_drv_desc      ; /* CAN      char drv interface     */
extern const Drv_desc gpio_drv_desc     ; /* GPIO     char drv interface     */
extern const Drv_desc drv_desc_usbitf   ; /* USB      char drv interface     */
extern const Drv_desc adc_drv_desc      ; /* ADC      char drv interface     */
extern const Drv_desc drv_desc_asi      ; /* ASI      char drv interface     */
extern const Drv_desc drv_desc_aud      ; /* AUD      char drv interface     */
extern const Drv_desc dac_drv_desc      ; /* DAC      char drv interface     */
extern const Drv_desc drv_desc_den      ; /* DEN      char drv interface     */
extern const Drv_desc drv_desc_dmx_small; /* DMX      char drv interface     */
extern const Drv_desc drv_desc_dmx_full ; /* DMX      char drv interface     */
extern const Drv_desc eth_drv_desc      ; /* ETH      char drv interface     */
extern const Drv_desc drv_desc_fre      ; /* FRE      char drv interface     */
extern const Drv_desc kbditf_drv_desc   ; /* KBDITF   char drv interface     */
extern const Drv_desc led_drv_desc      ; /* LED      char drv interface     */
extern const Drv_desc pow_drv_desc      ; /* POW      char drv interface     */
extern const Drv_desc drv_desc_sec      ; /* SEC      char drv interface     */
extern const Drv_desc drv_desc_smc      ; /* SMC      char drv interface     */
extern const Drv_desc drv_desc_swi      ; /* SWI      char drv interface     */
extern const Drv_desc drv_desc_vid      ; /* VID      char drv interface     */
extern const Drv_desc rad_drv_desc      ; /* RAD      char drv interface     */


/*
    3.4  Block driver interface
    ---------------------------
*/

extern const Drv_desc drv_desc_dsk_ata  ; /* DSK_ATA  block drv interface    */


/*
    4    Prototypes of public procedures
    ____________________________________
*/

#endif                                 /* End of BOARD_H                     */
