/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_asy.h"             */
/*  Constants and definitions for ASY driver.                                */
/*                                                                           */
/*  Author : JYA                        Last modification : 06/16/2023       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Bauds rates
            1.2  Stop bits
            1.3  Parity
            1.4  Other capabilities
            1.5  Register numbers

         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures
         5  RAM data to be inclueded in board_xyz.c


*/


#ifndef DRV_ASY_H                      /* If this file is not already        */
#define DRV_ASY_H                      /* included, include it.              */


/*
    1    Defines
    ____________

    1.1  Bauds rates
    ----------------
*/

#define BR_0                 0         /* Baudrate nul = Arret de l'ASY      */
#define BR_50                1         /* 50 Bauds                           */
#define BR_75                2         /* 75 Bauds                           */
#define BR_110               3         /* 110 Bauds                          */
#define BR_134               4         /* 134 Bauds                          */
#define BR_150               5         /* 150 Bauds                          */
#define BR_200               6         /* 200 Bauds                          */
#define BR_300               7         /* 300 Bauds                          */
#define BR_600               8         /* 600 Bauds                          */
#define BR_1200              9         /* 1200 Bauds                         */
#define BR_1800             10         /* 1800 Bauds                         */
#define BR_2400             11         /* 2400 Bauds                         */
#define BR_4800             12         /* 4800 Bauds                         */
#define BR_9600             13         /* 9600 Bauds                         */
#define BR_19200            14         /* 19200 Bauds                        */
#define BR_38400            15         /* 38400 Bauds                        */
#define BR_57600            16         /* 57600 Bauds                        */
#define BR_115200           17         /* 115200 Bauds                       */
#define BR_ALL  ((1<<(BR_115200+1))-1) /* Tous les modes supportes           */

/*
    1.2  Stop bits
    --------------
*/

#define ST_05                1         /* 0.5 stop bits                      */
#define ST_1                 2         /* 1 stop bits                        */
#define ST_15                4         /* 1.5 stop bits                      */
#define ST_2                 8         /* 2 stop bits                        */
#define ST_ALL             0xF         /* Tous les modes supportes           */


/*
    1.3  Parity
    -----------
*/

#define PAR_NONE             1         /* Pas de parite                      */
#define PAR_ODD              2         /* Parite ODD                         */
#define PAR_EVEN             4         /* Parite EVEN                        */
#define PAR_ALL            0x7         /* Tous les modes supportes           */



/*  1.4  Other capabilities
    -----------------------
*/

#define OTH_CTS           0x01         /* Hardware flow control support      */
#define OTH_FIFO          0x04         /* FIFO available                     */
#define OTH_CTSDMA        0x10         /* RTS/CTS handling done by DMA       */
#define OTH_DMA           0x40         /* DMA available                      */

#define ASY_NBREG           10
#define NBBLK                4
#define SZBLK              128

/*  1.5  Register numbers
    ---------------------
*/

#define NR_ASY_TYPE               0    /* string  TYPE                     1 */
#define NR_ASY_NAME               1    /* string  NAME                     2 */
#define NR_ASY_NUMDEV             3    /* integer NUM                      1 */
#define NR_ASY_LINE               4    /* integer BAUDS NBBITS PARITY      1 */
#define NR_ASY_SIG                5    /* integer CTS DSR RI DCD DTR RTS   1 */
#define NR_ASY_CHAR               6    /* integer CHAR1 CHAR2 CHAR3        1 */
#define NR_ASY_FRAME              7    /* integer MAXFRAME MINFRAME        1 */
#define NR_ASY_TEMPO              8    /* integer TMODE TEMPO              1 */
#define NR_ASY_FLOW               9    /* integer FLOWCTL BUFSIZE          1 */

/*
    2    Structures and typedefs
    ____________________________
*/

/* Structure de description d'une liaison asynchrone  .......................*/

typedef struct                         /* Hardware dependant procedures      */
  BEGIN                                /* ---------------------------------- */
    void         (*info)         ()  ; /* Get the UART hardware name         */
    void         (*open)         ()  ; /* Open one UART                      */
    void         (*close)        ()  ; /* Close one UART                     */
    void         (*setline)      ()  ; /* Write the UART line parameters     */
    void         (*setflow)      ()  ; /* Write flow control related values  */
    void         (*setsig)       ()  ; /* Sets DTR RTS TXBRK signal states   */
    void         (*getsig)       ()  ; /* Reads CTS DSR RI DCD RXBREAK states*/
    void         (*ittx)         ()  ; /* Enables/Disables TX FIFO IRQ       */
    IWORD        (*putbyte)      ()  ; /* Puts one byte in the TX FIFO       */
    IWORD        (*getbyte)      ()  ; /* Reads one byte from the RX FIFO    */
    void         (*setto)        ()  ; /* Starts an inactivity timer         */
    void         (*clearto)      ()  ; /* Stops the inactivity timer         */
                                       /*                                    */
    IWORD        (*snddma)       ()  ; /* Starts sending with DMA            */
    IWORD        (*rcvdma)       ()  ; /* Starts receiving with DMA          */
    IWORD        (*snddmastop)   ()  ; /* Aborts sending with DMA            */
    IWORD        (*rcvdmastop)   ()  ; /* Aborts receiving with DMA          */

  END            Asy_desc            ; /* ---------------------------------- */


typedef struct                         /* ASY Device Definition              */
  BEGIN                                /* ---------------------------------- */
    UINT         lctx                ; /* Low level device/context index     */
    UINT         supp_speed          ; /* Supported line rates               */
    UCHAR        supp_stop           ; /* Supported sto bit numbers          */
    UCHAR        supp_size[5]        ; /* Supported parity modes             */
    UCHAR        supp_other          ; /* Miscellaneous supported features   */
  END            Asy_drvdef          ; /* ---------------------------------- */

#define SZ_BUF     128

typedef struct                         /* ASY Device context                 */
  BEGIN                                /* ---------------------------------- */
    Req_ctx        rctx              ; /* The "Job" queues                   */
    Key_lnk        key               ; /* A single key by device             */
    Next_lnk       nlnk              ; /* References entite a droite         */
    const Asy_desc *desc             ;
    void           (*setsig)     ()  ; /* Set the RTS DTR TXBREAK sig. state */
    void           (*getsig)     ()  ; /* Get the CTS DSR RI DCD RXBREAK sig */
    void           (*ittx)       ()  ; /* Enable/Disable the TX interrupt    */
    IWORD          (*putbyte)    ()  ; /* Write a byte in the TX FIFO        */
    IWORD          (*getbyte)    ()  ; /* Read a byte from the RX FIFO       */
    void           (*setto)      ()  ; /* Start the CHAR/FRAME timer         */
    void           (*clearto)    ()  ; /* Stop the CHAR/FRAME timer          */
    IWORD          (*snddma)     ()  ; /* Start a UART DMA transmission      */
    IWORD          (*rcvdma)     ()  ; /* Start a UART DMA reception         */
    IWORD          (*snddmastop) ()  ; /* Stop a UART DMA sending            */
    IWORD          (*rcvdmastop) ()  ; /* Stop a UART DMA reception          */
                                       /*                                    */
    Asy_drvdef     *def              ; /* Definitions of the device          */
    UINT           lctx              ; /* Low-level device index             */
                                       /*                                    */
    IWORD          (*cb_send)    ()  ; /* End of asy_send() callback  UNUSED */
    IWORD          (*cb_rcv)     ()  ; /* End of reception callback   UNUSED */
    IWORD          (*cb_io)      ()  ; /* End of asy_io() callback    UNUSED */
    IWORD          (*cb_ind)     ()  ; /* IND_REPORT callback         UNUSED */
    INT            inihigh           ; /* Reference of what is stacked above */
                                       /*                                    */
    UINT           reg[ASY_NBREG]    ; /* Value of the registers             */
                                       /*                                    */
    USHORT         char1             ; /* End of message character 1         */
    USHORT         char2             ; /* End of message character 2         */
    USHORT         char3             ; /* End of message character 3         */
    USHORT         maxframe          ; /* Maximum frame size                 */
    USHORT         minframe          ; /* Minimum frame size                 */
    USHORT         tempo             ; /* Timeout (milliseconds)             */
    USHORT         bufsize           ; /* BUFSIZE                            */
    UCHAR          tmode             ; /* 0:NONE  1:CHARACTER  2:FRAME       */
    UCHAR          flowctl           ; /* 0:NONE  1:RTS/CTS  2:XON/XOFF      */
    UCHAR          dma               ; /* 0:byte by byte mode  1: DMA mode   */
                                       /*                                    */
    UCHAR          buf[SZ_BUF]       ; /* If no other buffer is available    */
    USHORT         ibuf              ; /* Index of the next byte to receive  */
                                       /*                                    */
    UCHAR          *snd              ; /* Address of the transmission buffer */
    USHORT         sndsize           ; /* Number of bytes to send            */
    USHORT         isnd              ; /* Index of the next byte to send     */
                                       /*                                    */
    UCHAR          *rcv              ; /* Addess of the reception buffer     */
    USHORT         rcvsize           ; /* Size of the reception buffer       */
    USHORT         ircv              ; /* Index of the next byte received    */
    USHORT         rcvcur            ; /* Count of current ongoing receives  */
                                       /*                                    */
    UCHAR          power             ; /* 1 if power is ON                   */
    UCHAR          stoptx            ; /* 1 if the outgoing flow is blocked  */
    UCHAR          stoprx            ; /* 1 if the incoming flow is blocked  */
    UCHAR          asyto             ; /* 1 if the UART timer is running     */
    UCHAR          sigin             ; /* b3:DCD  b2:RI  b1:DSR  b0:CTS      */
    UCHAR          sigout            ; /* b5:RTS  b4:DTR                     */
    UCHAR          txbreak           ; /* 1 if a TX BREAK is ongoing         */
    UCHAR          closing           ; /* 1 if a REQ_CLOSE has been received */
                                       /*                                    */
    UINT           cnt_tx            ; /* TX     IRQ count                   */
    UINT           cnt_rx            ; /* RX     IRQ count                   */
    UINT           cnt_sig           ; /* SIG    IRQ count                   */
    UINT           cnt_to            ; /* TO     IRQ count                   */
    UINT           cnt_brktx         ; /* TX BRK IRQ count                   */
    UINT           cnt_brkrx         ; /* RX BRK IRQ count                   */
    UINT           cnt_dmatx         ; /* TX DMA IRQ count                   */
    UINT           cnt_dmarx         ; /* RX DMA IRQ count                   */
  END            Asy_ctx             ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/

extern const Vmio_desc asy_vmio_desc ; /* ASY automaton description          */
extern UCHAR      asy_sysdrv         ; /* ASY driver number                  */
extern const UINT tasybauds[]        ; /* Baudrate corresponding to the      */
                                       /* baud rate index                    */

/*
    4    Prototypes of public procedures
    ____________________________________
*/

/* Procedures appelees par les controleurs d'interruption ...................*/

  void  itasyrx             (IWORD                                           );
  void  itasytx             (IWORD                                           );
  void  itasyto             (IWORD                                           );
  void  itasysig            (IWORD                                           );
  void  itasybrkrx          (IWORD                                           );
  void  itasybrktx          (IWORD                                           );
  void  itasydmarx          (IWORD,INT,INT                                   );
  void  itasydmatx          (IWORD                                           );


/* Procedures d'interface du driver systeme .................................*/

extern UCHAR asy_numdrv              ; /* Numero du driver ASY               */
extern UCHAR asy_sysdrv              ; /* Numero du driver ASY               */

  IWORD asy_get_rctx        (INT,Req_ctx**,Key_lnk**,Req_subctx**            );
  IWORD asy_send            (Req_ctx*,Req_subctx*,
                             UCHAR*,INT,INT,void(*)(),INT,INT,INT            );
  IWORD asy_receive         (Req_ctx*,Req_subctx*,
                             UCHAR*,INT,INT,INT,INT,INT                      );
  IWORD asy_setval          (Req_ctx*,Req_subctx*,UCHAR*,INT,INT,INT,INT     );
  IWORD asy_getval          (Req_ctx*,Req_subctx*,UCHAR*,INT,INT,INT,INT     );
  IWORD asy_io              (Req_ctx*,Req_subctx*,Job*,INT,INT               );

  IWORD asy_set_line        (INT, INT, INT, INT, UCHAR**,INT*                );
  IWORD asy_set_signals     (INT, INT, INT, UCHAR**,INT*                     );
  IWORD asy_set_eom         (USHORT *, INT, INT, INT, INT, UCHAR**,INT*      );
  IWORD asy_set_flowctl     (INT, INT, UCHAR**,INT*                          );


/* Procedures d'interface pour affichage console ............................*/

  IWORD asy_write           (INT, UCHAR *, INT                               );
  IWORD asy_read            (INT, UCHAR *, INT, INT *                        );

/* Interface procedure for statistical counters .............................*/

  IWORD get_asy_stats       (INT,
                             UINT*,UINT*,UINT*,UINT*,UINT*,UINT*,UINT*,UINT* );
  IWORD clear_asy_stats     (INT                                             );

/* Procedures de traces en memoire pour les bas niveaux (MEMLOG).............*/

  void  asy_log_init        (void                                            );
  void  asy_log             (INT,INT,INT                                     );
  void  asy_log_prt         (void                                            );


#endif                                 /* End of DRV_ASY_H.                  */
/*
    5  RAM data to be inclueded in board_xyz.c
    ___________________________________________
*/

#ifdef BOARD_XYZ_DATA_C

const USHORT  asy_nbiod = ASY_NBCTX  ; /* Nombre de devices pour le VMK.     */
const USHORT  asy_nblw  = ASY_NBCTX+1; /* Nombre de voies logiques           */

USHORT        asy_state [ASY_NBCTX+1]; /* Etat de voie logique               */
Asy_ctx       asy_ctx   [ASY_NBCTX]  ; /* Controller's context table         */

Itf_ctx asy_itfctx[1+ASY_NBCTX]      ; /*                                    */

#define ASY_BSIZE 40                   /* Biggest buffer with "asy_set_eom"  */
#define ASY_BCNT   8                   /*                                    */

#define ASY_SZ (ASY_BSIZE*ASY_BCNT)

UCHAR         asy_pool[ASY_SZ]             ;
UINT          asy_allbtm[(ASY_BCNT+31)/32] ;
const USHORT  asy_bsize = ASY_BSIZE        ;
const USHORT  asy_bcnt  = ASY_BCNT         ;

#endif                                 /* End of BOARD_XYZ_C                 */


