/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "os/include/stdtyp.l"           */
/*  Standard definitions and constants.                                      */
/*                                                                           */
/*  Author : JYA                        Last modification : 02/02/2024       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         Hyperpanel OS standard definitions and constants for C code
*/


#ifndef STDTYP_L                       /* Begin conditionnal compilation     */
#define STDTYP_L                       /* Identifier for this include        */

#include <vm.h>                        /* VM constants definitions           */

/* Instruction blocs ........................................................*/

                                       /* Blocks / structures markers ------ */
#define PROCEDURE                      /* A procedure definition follows     */
#define PROCTEST                       /* Unitary test only procedure        */
#define BEGIN            {             /* Begin instruction block or struct  */
#define END              }             /* End of structure definition        */
#define END_IF           }             /* End of "if"     block              */
#define END_ELSE         }             /* End of "else"   block              */
#define END_SW           }             /* End of "switch" block              */
#define END_FOR          }             /* End of "for"    block              */
#define END_WHILE        }             /* End of "while"  block              */
#define END_UNTIL        }             /* End of "until"  block              */
#define END_PROC         }             /* End of procedure                   */

                                       /* Logical / Binary operators ------- */
#define OR               |             /* Binary OR                          */
#define AND              &             /* Binary AND                         */
#define XOR              ^             /* Binary EXCLUSIVE OR                */
#define LOGOR            ||            /* Logical OR                         */
#define LOGAND           &&            /* Logical AND                        */
#define NOT              !             /* Logical NOT                        */

                                       /* Comparison operators ------------- */
#define EQ               ==            /* Equal                              */
#define NE               !=            /* Not equal                          */
#define GT               >             /* Strictly greater                   */
#define LT               <             /* Strictly lower                     */
#define GE               >=            /* Greater or equal                   */
#define LE               <=            /* Lower or equal                     */


/* Standard types .............................................................

   The HYPER_* types are "typedef" which can be found in <vm.h>
*/

typedef int              IWORD ;       /* CPU signed word                    */
typedef unsigned int     UIWORD;       /* CPU unsigned word                  */

#define CHAR             HYPER_CHAR    /* Signed integer        1 bytes      */
#define UCHAR            HYPER_UCHAR   /* Unsigned integer      1 bytes      */
#define SHORT            HYPER_SHORT   /* Signed integer        2 bytes      */
#define USHORT           HYPER_USHORT  /* Unsigned integer      2 bytes      */
#define INT              HYPER_INT     /* Signed integer        4 bytes      */
#define UINT             HYPER_UINT    /* Unsigned integer      4 bytes      */
#define LONG             HYPER_LONG    /* Signed integer      4/8 bytes      */
#define ULONG            HYPER_ULONG   /* Unsigned integer    4/8 bytes      */
#define FLOAT            HYPER_FLOAT   /* Floating point        4 bytes      */
#define DOUBLE           HYPER_DOUBLE  /* Floating point        8 bytes      */
#define OID              INT           /* oid (object identifier)            */

#define UNUSED_PARAM(x)  (void)(x)     /* Suppress Unused Parameter warnings.*/

typedef struct                         /* 64 bits float value.               */
  BEGIN                                /* ---------------------------------- */
    UINT            m1               ; /* b31     = sign bit                 */
                                       /* b30..20 = 11 exponent bits         */
                                       /* b19..0  = 20 mantissa MS bits      */
    UINT            m2               ; /* b31..0  = 32 mantissa LS bits      */
  END               FL64             ; /* ---------------------------------- */

typedef struct                         /* 64 bits unsigned integer value.    */
  BEGIN                                /* ---------------------------------- */
    UINT            high             ; /* 32 most  significant bits          */
    UINT            low              ; /* 32 least significant bits          */
  END               Uint64           ; /* ---------------------------------- */


/* Remarkable elementary data types values ..................................*/

#define IW0             (IWORD)0       /* La constante 0 format mot machine. */
#define IW1             (IWORD)1       /* La constante 1 format mot machine. */
#define IWN             (IWORD)-1      /* La constante -1 format mot machine.*/
#define HMAXUCHAR       (INT)0xFF      /* Valeur max. pour CHAR  non signe.  */
#define HMAXUSHORT      (INT)0xFFFF    /* Valeur max. pour SHORT non signe.  */
#define HMAXUINT             0xFFFFFFFF/* Valeur max. pour INT   non signe.  */
#define HMAXCHAR        (INT)0x7F      /* Valeur max. pour CHAR  signe.      */
#define HMAXSHORT       (INT)0x7FFF    /* Valeur max. pour SHORT signe.      */
#define HMAXINT         (INT)0x7FFFFFFF/* Valeur max. pour INT   signe.      */
#define HMAXLONG        HYPER_MAXLONG  /* Valeur max. pour LONG  signe.      */
#define HMAXULONG       HYPER_MAXULONG /* Valeur max. pour LONG  non signe.  */
#define CHLG            '\n'           /* Code ascii du line-feed.           */
#define STDTXT          (INT)1024      /* Longueur d'un buffer texte.        */
#define STDLINE         (INT)256       /* Longueur d'une ligne de caracteres.*/
#define NOTIMER         0xFFFFFFFF     /* Empty timer identifier             */


/* Standard type sizes in bytes..............................................*/

#define ZC     (INT)(sizeof(CHAR))     /* CHAR     size in bytes             */
#define ZS     (INT)(sizeof(SHORT))    /* SHORT    size in bytes             */
#define ZI     (INT)(sizeof(INT))      /* INT      size in bytes             */
#define ZL     (INT)(sizeof(LONG))     /* LONG     size in bytes             */
#define ZF     (INT)(sizeof(FLOAT))    /* FLOAT    size in bytes             */
#define ZD     (INT)(sizeof(DOUBLE))   /* DOUBLE   size in bytes             */
#define ZW     (INT)(sizeof(IWORD))    /* IWORD    size in bytes             */
#define LGAD   (INT)(sizeof(INT*))     /* Address  size in bytes             */
#define NBNS   (INT)5                  /* Nombre d'INT pour buffers noms     */


/*  Macros for warning messages .............................................*/

#define Warning0(a)              { warning0((INT)a);                       }
#define Warning1(a,b)            { warning1((INT)a,(IWORD)(b));            }
#define Warning2(a,b,c)          { warning2((INT)a,(IWORD)(b),(IWORD)(c)); }
#define Warning3(a,b,c,d)        { warning3((INT)a,(IWORD)(b),(IWORD)(c),  \
                                                   (IWORD)(d));            }
#define Warning4(a,b,c,d,e)      { warning4((INT)a,(IWORD)(b),(IWORD)(c),  \
                                                   (IWORD)(d),(IWORD)(e)); }
#define Warning5(a,b,c,d,e,f)    { warning5((INT)a,(IWORD)(b),(IWORD)(c),  \
                                                   (IWORD)(d),(IWORD)(e),  \
                                                   (IWORD)(f));            }

#define Warningt0(t,a)           { if (hmsg_flag&t) Warning0(a)            }
#define Warningt1(t,a,b)         { if (hmsg_flag&t) Warning1(a,b)          }
#define Warningt2(t,a,b,c)       { if (hmsg_flag&t) Warning2(a,b,c)        }
#define Warningt3(t,a,b,c,d)     { if (hmsg_flag&t) Warning3(a,b,c,d)      }
#define Warningt4(t,a,b,c,d,e)   { if (hmsg_flag&t) Warning4(a,b,c,d,e)    }
#define Warningt5(t,a,b,c,d,e,f) { if (hmsg_flag&t) Warning5(a,b,c,d,e,f)  }


/*  Macros for error messages ...............................................*/

#define Escap                    { retour = -1;                goto error; }
#define Setret(a)                { *ret   =  a;                goto error; }
#define Abort0(a)                { Warningt0(MERR,a)           Escap       }
#define Abort1(a,b)              { Warningt1(MERR,a,b)         Escap       }
#define Abort2(a,b,c)            { Warningt2(MERR,a,b,c)       Escap       }
#define Abort3(a,b,c,d)          { Warningt3(MERR,a,b,c,d)     Escap       }
#define Abort4(a,b,c,d,e)        { Warningt4(MERR,a,b,c,d,e)   Escap       }
#define Abort5(a,b,c,d,e,f)      { Warningt5(MERR,a,b,c,d,e,f) Escap       }
#define Error0(a)                { Warningt0(MERR,a)           Setret(a)   }
#define Error1(a,b)              { Warningt1(MERR,a,b)         Setret(a)   }
#define Error2(a,b,c)            { Warningt2(MERR,a,b,c)       Setret(a)   }
#define Error3(a,b,c,d)          { Warningt3(MERR,a,b,c,d)     Setret(a)   }
#define Error4(a,b,c,d,e)        { Warningt4(MERR,a,b,c,d,e)   Setret(a)   }
#define Error5(a,b,c,d,e,f)      { Warningt5(MERR,a,b,c,d,e,f) Setret(a)   }
#define War0(a)                  { Warningt0(MWAR,a)           Setret(a)   }


/*  Other macros ............................................................*/

#define Abs(a)          (((a)>=0)?(a):(-(a)))              /* Valeur absolue.*/

#ifndef nearint                                            /* Entier proche. */
#define nearint(a)      (((a)>=0)?(INT)((a)+0.5):(INT)((a)-0.5)) /*          */
#endif                                                     /*                */

#ifndef intround                                           /* Entier arrondi.*/
#define intround(a,b)   ((a+(a%b))/b)                      /*                */
#endif                                                     /*                */

#ifndef of7                                                /* Offset d'un    */
#define of7(type,champ) ((UINT)&(((type*)0)->champ))       /* champ dans une */
#endif                                                     /* structure.     */

#ifndef Min                                                /* Valeur min.    */
#define Min(a,b)        (((a)<=(b))?(a):(b))               /*                */
#endif                                                     /*                */

#ifndef Max                                                /* Valeur max.    */
#define Max(a,b)        (((a)>=(b))?(a):(b))               /*                */
#endif                                                     /*                */

#ifndef pow2                                               /* Mise au carre. */
#define pow2(a)         ((a)*(a))                          /*                */
#endif                                                     /*                */

#ifdef TRUE
#undef TRUE
#endif

#ifdef FALSE
#undef FALSE
#endif


/* Libraries fonctions prototypes ...........................................*/

#define Strlen(a)        (INT)strlen((char*)(a))
#define Strcpy(a,b)      strcpy((char*)(a),(char*)(b))
#define Strcmp(a,b)      strcmp((char*)(a),(char*)(b))

#define Memcpy(a,b,c)    memcpy((void*)(a),(void*)(b),(unsigned int)(c))
#define Memcmp(a,b,c)    memcmp((void*)(a),(void*)(b),(unsigned int)(c))
#define Memset(a,b,c)    memset((void*)(a),(int)(b),(unsigned int)(c))
#define Memccpy(a,b,c,d) memccpy((void*)(a),(void*)(b),(int)(c),\
                                              (unsigned int)(d))
#define Memmove(a,b,c)   memmove((void*)(a),(void*)(b),(unsigned int)(c))

#define Strcat(a,b)      strcat((char*)(a),(char*)(b))
#define Strncpy(a,b,c)   strncpy((char*)(a),(char*)(b),(unsigned int)(c))
#define Stricmp(a,b)     stricmp((char*)(a),(char*)(b))
#define Strncmp(a,b,c)   strncmp((char*)(a),(char*)(b),(unsigned int)(c))
#define Strnicmp(a,b,c)  strnicmp((char*)(a),(char*)(b),(unsigned int)(c))
#define Strncat(a,b,c)   strncat((char*)(a),(char*)(b),(unsigned int)(c))
#define Strchr(a,b)      (CHAR*)strchr((char*)(a),(int)b)
#define Strrchr(a,b)     (CHAR*)strrchr((char*)(a),(int)b)
#define Strstr(a,b)      (CHAR*)strstr((char*)(a),(char*)(b))
#define Strtol(a,b,c)    (INT)strtol((char*)(a),(char**)(b),(int)(c))

#define divfnt(a,b)      (((a)/(b)) + (((a) % (b)) != 0))


#endif                                 /* End of VH_H conditionnal compil    */
