/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/vmit.h"                */
/*  Constants and definitions for IT driver.                                 */
/*                                                                           */
/*  Author : JYA/FA                     Last modification : 08/31/2023       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Values fot the "next" field of Ithand_t
            1.2  Sotware IRQ reserved numbers

         2  Structures and typedefs
            2.1  Interrupt controller - Ithead_t
            2.2  Interrupt control handler - Ithand_t
            2.3  Low-level interrupt controller driver - Vmit_desc
            2.4  Grouping "Ithead_t" and "Ithand_t" - Vmit_def
            2.5  Data of an interrupt controller - Itdata_t
            2.6  Software raised IRQ context - Signal_ctx

         3  Public variables

         4  Prototypes of public procedures
            4.1  Procedures prototypes provided by "vmit.c"
            4.2  Low level procedures prototypes required by "vmit.c"

         5  RAM data to be included in board_xyz.c
         6  RAM data to be inclueded in space_iocont_xyz.c
*/


#ifndef VMIT_H                         /* If this file is not already        */
#define VMIT_H                         /* included, include it.              */

 
/*
    1    Defines
    ____________
*/

/* 1.1  Values fot the "next" field of Ithand_t .............................*/

#define SINGLE        0                /*                                    */
#define NEXT(a)       (a)              /*                                    */
#define DISPATCH(a)   (0x8000 |(a))    /*                                    */
#define ENDLINK       0xFFFF           /*                                    */
#define FASTIRQ       0xFF00           /*                                    */

/* 1.2  Sotware IRQ reserved numbers ........................................*/

#define VMIO_SOFTIRQ   0               /*                                    */
#define LPTICK_SOFTIRQ 1               /* Low priority Systick IRQ           */

/*
    2    Structures and typedefs
    ____________________________


    2.1  Interrupt controller - Ithead_t
    ------------------------------------
*/

typedef struct                         /* One entry in "ithead[]" table      */
  BEGIN                                /* ---------------------------------- */
    UINT        mask1                ; /* Bit corresponding to IT wire       */
    UINT        mask2                ; /* Second part of previous mask       */
    IWORD       hand                 ; /* Index in "ithand" table            */
  END           Ithead_t             ; /* ---------------------------------- */


/*
    2.2  Interrupt control handler - Ithand_t
    -----------------------------------------
*/

typedef struct                         /* One entry in "ithand[]" table      */
  BEGIN                                /* ---------------------------------- */
    UINT        next                 ; /* Next handler for the same IT       */
    IWORD       (*proc)()            ; /* IT processing procedure            */
    IWORD       nctx                 ; /* Context number                     */
    IWORD       ntype                ; /* Type                               */
  END           Ithand_t             ; /* ---------------------------------- */


/*
    2.3  Low-level interrupt controller driver - Vmit_desc
    ------------------------------------------------------
*/

typedef struct                         /* Low-level procedures interface     */
  BEGIN                                /* ---------------------------------- */
    void        (*init_vector)()     ; /* Procedure address "init_vector_xyz"*/
    void        (*end_vector )()     ; /* Procedure address "end_vector_xyz" */
    IWORD       (*intctl     )()     ; /* Procedure address "intctl_xyz"     */
    void        (*softname   )()     ; /* Procedure address "softname_xyz"   */
  END           Vmit_desc            ; /* ---------------------------------- */


/*
    2.4  Grouping "Ithead_t" and "Ithand_t" - Vmit_def
    --------------------------------------------------
*/

typedef struct                         /* Grouping structure                 */
  BEGIN                                /* ---------------------------------- */
    const IWORD    *idx              ; /* Index for levels in "ithead"       */
    const Ithead_t *head             ; /* Table address of fir IT treatment  */
    const Ithand_t *hand             ; /* Table address of handlers info     */
  END               Vmit_def         ; /* ---------------------------------- */


/*
    2.5  Data of an interrupt controller - Itdata_t
    -----------------------------------------------
*/

typedef struct                         /* Interruption data description      */
  BEGIN                                /* ---------------------------------- */
    IWORD      *adstack              ; /* Adress of stack to use             */
    IWORD       szstack              ; /* Stack size in bytes                */
    IWORD       flag                 ; /* Bits 0 of HPTR                     */
    IWORD       trigger              ; /* Trigger mode                       */
  END           Itdata_t             ; /* ---------------------------------- */

/*
    2.6  Software raised IRQ context - Signal_ctx
    ---------------------------------------------
*/

typedef struct                         /* Software raised IRQ context        */
  BEGIN                                /* ---------------------------------- */
    UCHAR       nb_dis               ; /* Nested calls to "dis_vmit_signal"  */
    UCHAR       pend_raise           ; /* Software IRQ rise is pending       */
    void        (*thproc)()          ; /* VMIT thread function address       */
    INT         thpar                ; /* VMIT thread function parameter     */
  END           Signal_ctx           ; /* ---------------------------------- */

/*
    3  Public variable
    __________________
*/

extern volatile     UCHAR itcurrprio ; /* Current IT priority                */
extern IWORD        itnext           ; /* Handler index                      */
extern IWORD        itdec[]          ; /* IT countdown                       */
extern UINT         itcnt[]          ; /* IT up-counters                     */
extern UCHAR        itpri[]          ; /* IT priority                        */
extern const USHORT int_nbirq        ; /* Size of ithand / itdec             */
extern const USHORT nb_softirq       ; /* Count of software raised IRQ's     */

extern const Sdrv_desc vmit_sdrv_desc; /* System driver description          */
extern const Vmit_desc vmit_desc_nvic ; /*                                    */

extern UINT  exception[]              ;



/*
    4    Prototypes of public procedures
    ____________________________________


    4.1  Procedures prototypes provided by "vmit.c"
    -----------------------------------------------
*/

IWORD init_vmit           (IWORD                                             );
IWORD end_vmit            (IWORD                                             );
void  int_proc            (UINT                                              );
IWORD get_vmit_stats      (IWORD(*)(),IWORD,IWORD,UINT*,UINT*,UINT*          );
IWORD clear_vmit_stats    (void                                              );

void  disIRQ              (void                                              );
void  enaIRQ              (void                                              );
IWORD getindIRQ           (IWORD (*)(),IWORD                                 );
IWORD ctlIRQ              (UINT,IWORD                                        );
IWORD chkIRQ              (void                                              );

IWORD vmit_thread         (INT,INT                                           );
IWORD create_vmit_thread  (CHAR*,void(*)(),INT,INT*                          );
IWORD delete_vmit_thread  (INT                                               );
void  raise_vmit_signal   (INT                                               );
void  clear_vmit_signal   (INT                                               );
void  dis_vmit_signal     (INT                                               );
void  ena_vmit_signal     (INT                                               );

IWORD alloc_hrt_timer     (CHAR*,INT*                                        );
IWORD free_hrt_timer      (INT                                               );
IWORD get_hrt_timer_par   (INT,UINT,UINT*,UINT*                              );
IWORD start_hrt_timer     (INT,UINT,UINT,INT,void(*)(),INT                   );
IWORD stop_hrt_timer      (INT                                               );

IWORD alloc_hrt_diginput  (CHAR*,INT,void(*)(),INT,INT*                      );
IWORD free_hrt_diginput   (INT                                               );
IWORD get_hrt_diginput    (INT                                               );
IWORD start_hrt_diginput  (INT                                               );
IWORD stop_hrt_diginput   (INT                                               );
IWORD alloc_hrt_digoutput (CHAR*,INT*                                        );
IWORD free_hrt_digoutput  (INT                                               );
IWORD set_hrt_digoutput   (INT,INT                                           );


/*
    4.2  Low level procedures prototypes required by "vmit.c"
    ---------------------------------------------------------
*/

void  enable_devirq       (void                                              );
void  disable_devirq      (void                                              );
void  set_vbr             (void*                                             );

#endif                                 /* End of VMIT_H                      */

/*
    5  RAM data to be included in board_xyz.c
    _________________________________________
*/

#ifdef BOARD_XYZ_DATA_C
const USHORT int_nbirq = INT_NBIRQ   ;

IWORD               itdec[INT_NBIRQ] ; /* Decompteurs d'ITs                  */
UINT                itcnt[INT_NBIRQ] ; /* IRQ counters                       */
UCHAR               itpri[INT_NBIRQ] ; /* IRQ priorities                     */

volatile Signal_ctx signal_ctx[NB_SOFTIRQ] ;
const USHORT   nb_softirq = NB_SOFTIRQ     ;

#endif                                 /* End of BOARD_XYZ_C                 */

/*  6  RAM data to be included in space_iocont_xyz.c
    _________________________________________________
*/

#ifdef SPACE_IOCONT_XYZ_C

/* Linker defined symbols ...................................................*/

extern IWORD     stack_irq_l         ; /* Ad. basse des piles d'IT           */
extern IWORD     stack_irq_h         ; /* Ad. haute des piles d'IT           */

/* Table of interrupt stack start addresses ................................ */

const IWORD *itstk[8] =                /* Table des adresses des piles d'IT  */
  BEGIN                                /* ---------------------------------- */
    &stack_irq_l + OF0/ZW           ,  /* Pile pour IT de priorite 0         */
    &stack_irq_l + OF1/ZW           ,  /* Pile pour IT de priorite 1         */
    &stack_irq_l + OF2/ZW           ,  /* Pile pour IT de priorite 2         */
    &stack_irq_l + OF3/ZW           ,  /* Pile pour IT de priorite 3         */
    &stack_irq_l + OF4/ZW           ,  /* Pile pour IT de priorite 4         */
    &stack_irq_l + OF5/ZW           ,  /* Pile pour IT de priorite 5         */
    &stack_irq_l + OF6/ZW           ,  /* Pile pour IT de priorite 6         */
    &stack_irq_l + OF7/ZW           ,  /* Pile pour IT de priorite 7         */
  END                                ; /* ---------------------------------- */

/* Table of interrupt stack end addresses .................................. */

const IWORD *endstk[8] =               /* Table des adresses fin de piles    */
  BEGIN                                /* ---------------------------------- */
    &stack_irq_l + OF0/ZW - SZ0/ZW  ,  /* Pile pour IT de priorite 0         */
    &stack_irq_l + OF1/ZW - SZ1/ZW  ,  /* Pile pour IT de priorite 1         */
    &stack_irq_l + OF2/ZW - SZ2/ZW  ,  /* Pile pour IT de priorite 2         */
    &stack_irq_l + OF3/ZW - SZ3/ZW  ,  /* Pile pour IT de priorite 3         */
    &stack_irq_l + OF4/ZW - SZ4/ZW  ,  /* Pile pour IT de priorite 4         */
    &stack_irq_l + OF5/ZW - SZ5/ZW  ,  /* Pile pour IT de priorite 5         */
    &stack_irq_l + OF6/ZW - SZ6/ZW  ,  /* Pile pour IT de priorite 6         */
    &stack_irq_l + OF7/ZW - SZ7/ZW  ,  /* Pile pour IT de priorite 7         */
  END                                ; /* ---------------------------------- */

#endif                                 /* End of SPACE_IOCONT_XYZ_H          */
