/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/vmk.h"                 */
/*  Constants and definitions for VMK's.                                     */
/*                                                                           */
/*  Author : JYA/FA                     Last modification : 07/16/2023       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  VMK waiting queues definitions and numbering
            1.2  VMK tables sizing
            1.3  VMK other constants
            1.4  Task_system structure field values
            1.5  VMK identifiers templates
            1.6  Values of the "nuser" global variable

         2  Structures and typedefs
            2.1  Internal queue - Iq_vmk
            2.2  Queue proiority - Iq_prio
            2.3  Awaited of event list context - Waitevtlist
            2.4  Awaited event descriptor - Waitevt
            2.5  Event filter context - Iq_filt
            2.6  Time-out structure for VMK - S_to
            2.7  Route structure for VMK - Route
            2.8  Conditional event sending structure for VMK - Sess_filt
            2.9  Tasks group description structure - Task_grp
            2.10 Context task description structure - Task_ctx
            2.11 System task description structure - Task_system
            2.12 Context task semaphore description structure - Task_sem
            2.13 Context owner description structure - S_owner

         3  Public variables
            3.1  Defined in space_appcont_xyz.c
            3.2  Defined in vmk1.c

         4  Prototypes of public procedures
            4.1  Procedures prototypes for VMK main loop
            4.2  Procedures prototypes for automaton
            4.3  Procedures prototypes for waiting queue
            4.4  Procedures prototypes for conditions on sessid
            4.5  Procedures prototypes for routing
            4.6  Procedures prototypes for temporisations
            4.7  Procedures prototypes for task and stack handling
            4.8  Procedures prototypes for semaphore
            4.9  Procedures prototypes for owner
            4.10 Procedures prototypes for miscelleanous
            4.11 Procedures prototypes for software debugger
            4.12 Procedures prototypes for VMK lower layer

         5  RAM data to be included in space_appcont_xyz.c
            5.1  FSM's
            5.2  VMK internal queues
            5.3  VMK routes
            5.4  VMK timers
            5.5  VMK tasks
            5.6  VMK semaphores
            5.7  VMK owners
            5.8  VMK stacks
*/


#ifndef VMK_H                          /* If this file is not already        */
#define VMK_H                          /* included, include it.              */


/*
    1    Defines
    ____________


    1.1  VMK waiting queues definitions and numbering
    -------------------------------------------------
*/

#define QNB_FREEEVT       0        /* Empty entries queue                    */
#define QNB_DBG_P0        1        /* Debugger events                        */
#define QNB_AUT_FIRST     2        /* VMK FSM events - first                 */
#define QNB_AUT_LAST      5        /* VMK FSM events - last                  */
#define QNB_TSK_FIRST     6        /* VMK tasks events - first               */
#define QNB_TSK_LAST     21        /* VMK tasks events - last                */
#define QNB_TSK_KILL     22        /* TASK_KILL events only                  */
#define QNB_SCR_P2       23        /* Scripts events - High priority         */
#define QNB_PAN_P2       24        /* Panels  events - High priority         */
#define QNB_PAN_P1       25        /* Panels  events - Medium priority       */
#define QNB_SCR_P1       26        /* Scripts events - Medium priority       */
#define QNB_SCR_P0       27        /* Scripts events - Low    priority       */
#define QNB_PAN_P0       28        /* Panels  events - Low    priority       */

#define PRIO_DBG_P0       0        /* Debugger events                        */
#define PRIO_AUT_MAX      1        /* VMK FSM events - max                   */
#define PRIO_AUT_MIN      2        /* VMK FSM events - min                   */
#define PRIO_TSK_MAX      3        /* VMK TSK events - max                   */
#define PRIO_TSK_MIN     18        /* VMK TSK events - min                   */
#define PRIO_TSK_KILL    19        /* TASK_KILL events only                  */
#define PRIO_SCR_P2      20        /* Scripts events - High priority         */
#define PRIO_PAN_P2      21        /* Panels  events - High priority         */
#define PRIO_PAN_P1      22        /* Panels  events - Medium priority       */
#define PRIO_SCR_P1      23        /* Scripts events - Medium priority       */
#define PRIO_SCR_P0      24        /* Scripts events - Low    priority       */
#define PRIO_PAN_P0      25        /* Panels  events - Low    priority       */

#define PRIO_AUT_NB       2        /* VMK FSM events - Count of priorities   */
#define PRIO_TSK_NB      16        /* VMK TSK events - Count of priorities   */

#define PRIO_FSM          2


/* 
    1.2  VMK tables sizing
    ----------------------
*/

#define NLIMEVT           4        /* Max number of limit structures         */
#define MAX_INDEX        10        /* Max number of routing events procedure */

#define IQ_VMK_SZ        29        /* Size of "the iq_vmk[]" table           */
#define IQ_PRIO_SZ       26        /* Size of the "iq_prio[]" table          */


/*
    1.3  VMK other constants
    ------------------------
*/

#define STCYCLE         200        /* Cycle time in milliseconds             */
#define SMINQUANT        10        /* Minimum quantum in milliseconds        */


/*
    1.4  Task_system structure field values
    ---------------------------------------
*/

/* Values for the "use" field ...............................................*/

#define TU_FREE           0       /* The context us unused, it is free       */
#define TU_RT             1       /* The context is used for a real-time task*/
#define TU_STD            2       /* The context is used for a standard task */

/* Values for the "state" field .............................................*/

#define TT_FREE           0        /* Task context is free                   */
#define TT_NONE           1        /* No treatments in progress              */
#define TT_RUN            2        /* Treatment is running                   */
#define TT_READY          3        /* Treatment is waiting for CPU           */
#define TT_WAIT           4        /* Treatment is waiting for event         */

/*
    1.5  VMK identifiers templates
    ------------------------------
*/

#define TAUT     0x10FF0000        /* FSM identifier                         */
#define TEVL     0x11FFFF00        /* Wait event list identifier             */
#define TRTE     0x12FFFF00        /* Route identifier                       */
#define TTIM     0x13000000        /* Timer identifier                       */
#define TTSK     0x14FFFF00        /* TAsk identifier                        */
#define TSEM     0x15FFFF00        /* Semaphore identifier                   */
#define TOWN     0x16FFFF00        /* Owner identifier                       */

#define MAUT     0x0000FFFF        /* FSM identifier mask                    */
#define MEVL     0x000000FF        /* Wait event list identifier mask        */
#define MRTE     0x000000FF        /* Route identifier mask                  */
#define MTIM     0x00FFFFFF        /* Timer identifier mask                  */
#define MTSK     0x000000FF        /* TAsk identifier mask                   */
#define MSEM     0x000000FF        /* Semaphore identifier mask              */
#define MOWN     0x000000FF        /* Owner identifier mask                  */

/*
    1.6  Values of the "nuser" global variable
    -------------------------------------------
*/
#define U_INI             0            /* "init_vmk" code                    */
#define U_VMK             1            /* "vmk" code                         */
#define U_FSYS            2            /* System FSM treatment code          */
#define U_FSTD            3            /* Standard FSM treatment code        */
#define U_FUSR            4            /* User FSM treatment code            */
#define U_TSK             5            /* User task code                     */
#define U_SCR             6            /* Script or directly called proc     */

/* The "nuser" never takes this U_ICC value, which is used only to build ......
   the mask of the "chk_user_rights" procedure
*/
#define U_ICC             7            /* Indirectly called C procedure      */

/*
    2    Structures and typedefs
    ____________________________


    2.1  Internal queue - Iq_vmk
    ----------------------------
*/

typedef struct                         /* One VMK internal queue             */
  BEGIN                                /* ---------------------------------- */
    USHORT      evcnt                ; /* Count of queued events             */
    USHORT      first                ; /* First queued event index           */
    USHORT      last                 ; /* Last queued event index            */
                                       /*                                    */
    UCHAR       qnext                ; /* Next queue, forward index          */
    UCHAR       qprev                ; /* Previous queue, backward index     */
    UCHAR       prio                 ; /* Queue priority                     */
                                       /*                                    */
    UCHAR       hfilt                ; /* Index 1st "iq_filt" structure      */
    USHORT      unevcnt              ; /* Count of unlocked events           */
  END           Iq_vmk               ; /* ---------------------------------- */


/*
    2.2  Queue proiority - Iq_prio
    ------------------------------
*/

typedef struct                         /* One queue priority                 */
  BEGIN                                /* ---------------------------------- */
    USHORT      evcnt                ; /* Sum of events for all the queues   */
    UCHAR       qnext                ; /* Next queues index                  */
    UCHAR       qcnt                 ; /* Count of queues for the priority   */
  END           Iq_prio              ; /* ---------------------------------- */


/*
    2.3  Awaited of event list context - Waitevtlist
    ------------------------------------------------
*/

typedef struct                         /* Awaited event list context         */
  BEGIN                                /* ---------------------------------- */
    USHORT      ownidx               ; /* Owner index/number                 */
    UCHAR       mode                 ; /* Freeing mode 0:manual 1:automatic  */
    USHORT      aut                  ; /* Automaton which created this list  */
    USHORT      lw                   ; /* LW        which created this list  */
    USHORT      evcnt                ; /* Count of awaited events            */
    USHORT      first                ; /* First descriptor index             */
  END           Waitevtlist          ; /* ---------------------------------- */


/*
    2.4  Awaited event descriptor - Waitevt
    ---------------------------------------
*/

typedef struct                         /* One awaited event descriptor       */
  BEGIN                                /* ---------------------------------- */
    UCHAR       typ                  ; /* Indicates the contents of val1,val2*/
    INT         val1                 ; /* First descriptor value             */
    INT         val2                 ; /* Second descriptor value            */
    USHORT      next                 ; /* Next decsriptor index              */
  END           Waitevt              ; /* ---------------------------------- */


/*
    2.5  Event filter context - Iq_filt
    -----------------------------------
*/

typedef struct                         /* Event filter event                 */
  BEGIN                                /* ---------------------------------- */
    USHORT      ownidx               ; /* Owner index/number                 */
    UCHAR       qnb                  ; /* Queue the filter is associated with*/
    UCHAR       sat                  ; /* Indicates if the filter matchs     */
    USHORT      aut                  ; /* Automaton that locks the queue     */
    USHORT      lw                   ; /* LW that locks the queue            */
    USHORT      next                 ; /* Next context, forward index        */
    USHORT      waitidx              ; /* Events list context index          */
  END           Iq_filt              ; /* ---------------------------------- */


/*
    2.6  Time-out structure for VMK - S_to
    --------------------------------------

    The table of running timers in the VMK is an array of "S_to" structures.
    There is one "S_to" structure per current time-out.
*/

typedef struct                         /* An entry in the TIME-OUT table     */
  BEGIN                                /* ---------------------------------- */
    USHORT      ownidx               ; /* Owner index/number                 */
    UINT        ident                ; /* Timer order number (24 bits)       */
    UINT        duration             ; /* Timer duration in milliseconds     */
    UCHAR       qnb                  ; /* Waiting queue number               */
    UCHAR       flags                ; /* 1 if armed by set_uto()            */
                                       /*                                    */
    USHORT      aut                  ; /* Num of automaton receiving the TO  */
    USHORT      lw                   ; /* LW number of receiving automation  */
    USHORT      code                 ; /* Event code to send at time-out     */
    UINT        res                  ; /* Reserve code value                 */
                                       /*                                    */
    USHORT      link                 ; /* Pointer to the next running timer  */
    USHORT      ticksh               ; /* Expiration date in ticks - MS bits */
    UINT        ticksl               ; /* Expiration date in ticks - LS bits */
  END           S_to                 ; /* ---------------------------------- */


/*
    2.7  Route structure for VMK - Route
    ------------------------------------

    The VMK event routing table is a table where each entry is a "Route"
    structure.
*/

typedef struct                         /* A route in the routing table       */
  BEGIN                                /* ---------------------------------- */
    USHORT      ownidx               ; /* Owner index/number                 */
    USHORT      aut                  ; /* Destination automaton number   (E) */
    USHORT      lw                   ; /* Destination logical way        (E) */
    USHORT      saut                 ; /* Sending automaton number       (E) */
    USHORT      slw                  ; /* Sending logical way            (E) */
                                       /*                                    */
    USHORT      code1                ; /* Minimun event code             (E) */
    USHORT      code2                ; /* Maximum event code             (E) */
    UINT        res1                 ; /* Minimum reserve field value    (E) */
    UINT        res2                 ; /* Maximum reserve field value    (E) */
                                       /*                                    */
    USHORT      flags                ; /* Priority +Grab +Normal/Default (E) */
    USHORT      next                 ; /* Chaining to the next route         */
                                       /*                                    */
    UINT        ident                ; /* Route identifier _WORK_            */
  END           Route                ; /* ---------------------------------- */


/*
    2.8  Conditional event sending structure for VMK - Sess_filt
    ------------------------------------------------------------

    The VMK event sending conditions table is a table where each entry is a
    "Sess_filt" structure.
*/

typedef struct                         /* An entry in "sess_cond" table      */
  BEGIN                                /* ---------------------------------- */
    UINT        iod                  ; /* IOD of events source               */
    USHORT      code                 ; /* Value of the event                 */
    USHORT      sessid               ; /* Current session number             */
    USHORT      act                  ; /* Field of actions to do             */
  END           Sess_filt            ; /* ---------------------------------- */
 

/*
    2.9  Tasks group description structure - Task_grp
    -------------------------------------------------

    The static table of task group description "task_group[]", which is
    defined in "vmk1.c", is an array of "Task_grp" structures.
*/

typedef struct                         /* An entry in "task_group" table     */
  BEGIN                                /* ---------------------------------- */
    const CHAR *group                ; /* Name of the task group             */
    UCHAR       use                  ; /* 1:real time, 2:standard            */
    UCHAR       autostart            ; /* 0:application 1:init_vmk           */
    IWORD       (*procstart)()       ; /* Starting procedure of the group    */
  END           Task_grp             ; /* ---------------------------------- */


/*
    2.10 Context task description structure - Task_ctx
    --------------------------------------------------

    The task context table "vmk_task[]", which is defined in the file
    "hypspace.h", is an array of "Task_ctx" structures.

    Note: Assembler procedures for task switching use the size of this
    structure and the offset of the "proc" and "par" fields. Any modification
    of this structure must be reported in the "exec_task" procedures of the
    different hardware.
*/

typedef struct                         /* A task context                     */
  BEGIN                                /* ---------------------------------- */

/* Table description ------------------------------------------------------- */

    UINT           *bpile            ; /* Stack base address                 */
    UINT            tpile            ; /* Size of the stack                  */
    void           *desc             ; /* Description code/DATA/BSS          */
    IWORD          (*freeproc)()     ; /* Freeing procedure address          */

/* Execution of a transition by the automaton engine ------------------------*/

    const S_trans  *tra              ; /* Transition address                 */
    IWORD          (*proc)()         ; /* Entry procedure address            */
    INT             par              ; /* Paremeter for "proc" call          */
    USHORT          condition        ; /* Modified by "relfsm"               */

/* Used by "schedule" ------------------------------------------------------ */

    UCHAR           use              ; /* Flag for current use of this ctx   */
    UCHAR           group            ; /* Group number of this task          */
    UCHAR           state            ; /* Current treamtent status           */
    UCHAR           nintq            ; /* Queue from which the event comes   */
    UINT            minquantum       ; /* Minimum quantum duration in msec   */
    UINT            endquantum       ; /* Current quantum ending date        */

/* Used by user tasks for the LW of TASK ----------------------------------- */

    USHORT          ownidx           ; /* Owner index/number                 */
    IWORD          (*entry)(void*)   ; /* Entry procedure address            */
    void           *param            ; /* Parameter for the procedure        */
    S_evt           currentevt       ; /* Current event backup               */
    UINT            toid             ; /* Current timer id or 0 if no timer  */
    USHORT          idx              ; /* Allocated stack index, or FFFFF    */
    UCHAR           qnb              ; /* Waiting queue number 2..15         */
    UCHAR           isem             ; /* Index of waited semaphore          */
    UCHAR           next             ; /* Next      waiting for the semaphore*/
    UCHAR           prev             ; /* Previous  waiting for the semaphore*/
    UCHAR           started          ; /* "start_task" counter               */
  END           Task_ctx             ; /* ---------------------------------- */


/*
    2.11 System task description structure - Task_system
    ----------------------------------------------------
*/

typedef struct                         /* System task context                */
  BEGIN                                /* ---------------------------------- */
    IWORD         *sp                ; /* 0x0000 Stack pointer               */
    void         (*ip)()             ; /* 0x0004 Instruction pointer         */
  END              Task_system       ; /* ---------------------------------- */


/*
    2.12 Context task semaphore description structure - Task_sem
    ------------------------------------------------------------

    The semaphore context table "task_sem[]", which is defined in the file
    "hypspace.h", is an array of "Task_sem" structures.
*/

typedef struct                         /* A semaphore                        */
  BEGIN                                /* ---------------------------------- */
    USHORT          ownidx           ; /* Owner number/index                 */
    SHORT           counter          ; /* Counter current value              */
    UCHAR           first            ; /* First pending task                 */
    UCHAR           last             ; /* Last pending task                  */
    UCHAR           qnb              ; /* Queue where TASK_SIGNAL sent       */
  END               Task_sem         ; /* ---------------------------------- */


/*
    2.13 Context owner description structure - S_owner
    --------------------------------------------------
*/

typedef struct                         /* A owner                            */
  BEGIN                                /* ---------------------------------- */
    UINT            ident            ; /* FSM or task identifier             */
    UCHAR           father           ; /* Father node number                 */
    UCHAR           elder            ; /* Elder son node number              */
    UCHAR           left             ; /* left/previous brother node         */
    UCHAR           right            ; /* right/next brother node            */
                                       /* ---------------------------------- */
    UINT            memuid           ; /* Memory user to be used             */
  END               S_owner          ; /* ---------------------------------- */

/*
    3    Public variables
    _____________________


    3.1  Defined in space_appcont_xyz.c
    -----------------------------------
*/

/* Constant variables .......................................................*/

extern const UCHAR  vmk_aut_max      ; /* Maximum number of VMK FSM's        */
extern const USHORT vmk_iq_evt_max   ; /* Size of "vmk_iq_evt[]"             */
extern const UCHAR  vmk_route_max    ; /* Maximum number of routes           */
extern const USHORT vmk_sess_filt_max; /* Maximum number of conditions       */
extern const UCHAR  vmk_sem_max      ; /* Maximum number of task semaphores  */
extern const UCHAR  vmk_to_max       ; /* Maximum number of VMK timers       */
                                       /*                                    */
extern const UCHAR  vmk_task_max     ; /* Maximum number of tasks            */
extern const UCHAR  nb_stack_sz      ; /* Number of sizes of stacks          */
extern const UCHAR  vmk_owner_max    ; /* Maximum number of owners           */

/* Tables ...................................................................*/

extern Aut_ctx      vmk_aut_ctx  []  ; /* Declared FSM set"s table           */
extern S_evt        vmk_iq_evt   []  ; /* VMK internal queues shared storage */
extern UCHAR        vmk_evt_lock []  ; /* Locks   of internal queues of VMK  */
extern Route        vmk_route    []  ; /* Global routes table                */
extern Sess_filt    vmk_sess_filt[]  ; /* Global conditions table            */
extern Task_sem     vmk_sem      []  ; /* Global semaphore table             */
extern S_to         vmk_to       []  ; /* Time-out structure table           */

extern UINT         used_stack   []  ; /* Stacks allocations table           */
extern Task_ctx     vmk_task     []  ; /* Task context table                 */
extern USHORT       task_state   []  ; /* Task state table                   */
extern UCHAR        task_pstack  []  ; /* Stack pointers table               */
extern S_owner      vmk_owner[]      ; /* Onwner table                       */

/*
    3.2  Defined in vmk1.c
    ----------------------
   
   System FSM numbers .........................................................

   The VMK itself, using "add_aut" from "init_vmk", declares the TASK FSM.
   its number, its index in the "vmk_aut[]" table, is kept in the "aut_task"
   global variable
*/

extern UCHAR         aut_task        ; /* TASK   Automaton number            */
extern UCHAR         aut_script      ; /* SCRIPT Automaton number            */
extern UCHAR         aut_panel       ; /* PANEL  Automaton number            */

/* Statistical counters ......................................................
*/

extern  UINT         cnt_putevt_fae  ; /* Count of calls to "putevt_fae"     */
extern  UINT         cnt_putevt_vmk  ; /* Count of calls to "putevt_vmk"     */
extern  UINT         cnt_vmk_trans   ; /* Count of VMK cycles                */


/* VMK running flags ..........................................................
*/

extern UCHAR        vmos_run         ; /* VMOS running flag                  */
extern UCHAR        idle_sleep       ; /* Idle mode status                   */
extern USHORT       evt_filter       ; /* External events driver filtering   */


/* Current event for all VMK user tasks ......................................

   This is a copy of the last extracted event for FSM_TASK. This copy is
   shared amongs all tasks and therefore it contents may change each time
   a task is unscheduled, in other words the contents may not be the same
   before and after any blocking call.
*/

extern S_evt        task_evt         ; /* Current event of FSM_TASK          */


/* For assembly "exec_task" and "return_atsk" coded functions .................

   Importing structure sizes, such as "sizeof(Task_ctx)" or structure fields
   offsets is not easy for assembly written functions, such as "exec_task"
   and "return_task". The most easy way is to define const variables containing
   those integers and then use the const variables from ASM code. Here we
   define the "Task_ctx" structure size in bytes and the two structure offsets
   for the "proc" and "par" fields of the "Task_ctx" structure.
*/

extern const UINT vmk_task_sz        ; /* sizeof(Task_ctx)                   */
extern const UINT vmk_task_of7_proc  ; /* of7(Task_ctx,proc)                 */
extern const UINT vmk_task_of7_par   ; /* of7(Task_ctx,par)                  */

/*
    4    Prototypes of public procedures
    ____________________________________
*/


/*
    4.1  Procedures prototypes for VMK main loop
    --------------------------------------------
*/

IWORD init_vmk            (CHAR*,CHAR*,UINT*,INT*                            );
IWORD vmk                 (UINT,S_evt*,UINT*,UINT*                           );
IWORD end_vmk             (void                                              );


/*
    4.2  Procedures prototypes for automaton
    ----------------------------------------
*/

IWORD add_aut               (CHAR *,const S_fsm *const((*)[]),const USHORT[],
                            UINT, USHORT[],UINT, UCHAR[],UINT, S_evt*,UINT*  );
IWORD add_uaut              (CHAR *,const S_fsm *const ((*)[]),const USHORT[],
                             UINT, USHORT[],UINT, UCHAR[],UINT, S_evt*,UINT* );
IWORD add_saut              (CHAR *,const S_fsm *const((*)[]),const USHORT[],
                             UINT, USHORT[],UINT, UCHAR[],UINT, S_evt*,UINT* );
IWORD get_aut_id            (CHAR*,UINT*,INT*                                );
void  rel_aut               (UINT                                            );
IWORD push_evt_aut          (S_evt*                                          );
IWORD putevt_vmk            (UINT,S_evt*                                     );
IWORD waitevt_vmk           (UINT,UINT,UINT,UINT,S_evt*                      );
IWORD unlockevt_vmk         (UINT,UINT,UINT                                  );
IWORD clearevt_vmk          (UINT,UINT,UINT,UINT,UINT*,UINT*                 );
IWORD sendevt_aut           (UINT,UINT,UINT,UINT,UCHAR*,INT,UINT,UINT,UINT   );
IWORD get_nintq             (void                                            );
IWORD get_naut              (void                                            );
IWORD get_nlw               (void                                            );
IWORD get_codeevt           (void                                            );
const S_evt* get_currentevt (void                                            );
IWORD get_nfsm              (void                                            );
IWORD get_nctx              (void                                            );
IWORD get_nuser             (void                                            );
IWORD get_nownidx           (void                                            );

IWORD alloc_waitevtlist     (UINT,UINT*                                      );
IWORD free_waitevtlist      (UINT                                            );
IWORD add_waitevt           (UINT,UINT,UINT,UINT                             );
IWORD del_waitevt           (UINT,UINT,UINT,UINT                             );
IWORD clear_waitevtlist     (UINT                                            );
IWORD nb_waitevtlist        (UINT,INT*                                       );


/*
    4.3  Procedures prototypes for waiting queue
    --------------------------------------------
*/

void  init_fae            (void                                              );
void  free_fae            (INT,INT                                           );
INT   waitevt_fae         (UINT,UINT,INT*,INT*                               );
INT   ldr_waitevt_fae     (INT,INT,UINT*,S_evt**,INT*,INT*                   );


/*
    4.4  Procedures prototypes for conditions on sessid
    ---------------------------------------------------
*/

IWORD add_sessid_filt     (UINT,UINT,UINT                                    );
IWORD mod_sessid_filt     (UINT,UINT,USHORT*                                 );
IWORD del_sessid_filt     (UINT,UINT                                         );
IWORD get_sessid          (UINT,UINT,USHORT*                                 );
IWORD put_event           (S_evt*                                            );
IWORD del_evtsess         (UINT,UINT                                         );


/*
    4.5  Procedures prototypes for routing
    --------------------------------------
*/

IWORD add_route           (UINT,UINT,UINT,UINT,UINT,UINT,UINT,UINT,UINT,UINT*);
IWORD list_route          (UINT,UINT,UINT,UINT,UINT,UINT,UINT,UINT,UINT,
                           UINT*,UINT,UINT*                                  );
IWORD del_route           (UINT*,UINT                                        );
IWORD set_route_flags     (UINT*,UINT,UINT,UINT                              );
IWORD set_route_iod       (UINT*,UINT,UINT                                   );
IWORD route_event         (S_evt*,UINT*                                      );
IWORD add_uroute          (UINT,UINT,UINT,UINT,UINT,UINT*                    );
IWORD list_uroute         (UINT,UINT,UINT,UINT,UINT,UINT*,UINT,UINT*         );
IWORD del_uroute          (UINT*,UINT                                        );
IWORD route_uevent        (UINT,UINT,UCHAR*,INT,UINT,UINT,UINT,UINT*         );
IWORD del_evtroute        (UINT,UINT*                                        );
IWORD get_route           (UINT,Route*                                       );
IWORD status_route        (UINT,CHAR*,UINT,CHAR*,UINT                        );
IWORD get_event_routing   (INT,CHAR**,CHAR**,INT*,INT*,INT*,CHAR**           );


/*
    4.6  Procedures prototypes for temporisations
    ---------------------------------------------
*/

IWORD time_init           (                                                  );
void  time_itmsec         (UINT                                              );
IWORD set_to              (UINT,UINT,UINT,UINT,UINT,UINT,UINT,UINT*          );
IWORD clear_to            (UINT,UINT,UINT                                    );
IWORD set_uto             (UINT,UINT,UINT,UINT,UINT*                         );
IWORD clear_uto           (UINT                                              );
IWORD set_tto             (UINT,UINT,UINT,UINT,UINT*                         );
IWORD clear_tto           (UINT                                              );
IWORD clear_fsmlwto       (UINT,UINT                                         );


/*
    4.7  Procedures prototypes for task and stack handling
    ------------------------------------------------------
*/

IWORD create_task         (CHAR*,IWORD(*)(void*),IWORD(*)(void),UINT*,
                           UINT,UINT,void*,UINT,UINT*                        );
IWORD delete_task         (UINT                                              );
IWORD start_task          (UINT                                              );
IWORD suspend_task        (INT                                               );
IWORD exit_task           (void                                              );
IWORD get_task_id         (UINT*                                             );
IWORD waitevt_task        (UINT[][3],UINT,UINT,UINT                          );
IWORD readevt_task        (UINT*,UINT                                        );
IWORD sendevt_task        (UINT,UINT,UINT,UCHAR*,INT,UINT,UINT,UINT          );
IWORD list_task           (UINT,UINT*,CHAR*,INT*                             );


/*
    4.8  Procedures prototypes for semaphore
    ----------------------------------------
*/

IWORD create_semaphore    (INT,UINT*                                         );
IWORD delete_semaphore    (UINT,INT*                                         );
IWORD put_semaphore       (UINT                                              );
IWORD get_semaphore       (UINT,INT*                                         );


/*
    4.9  Procedures prototypes for owner
    ------------------------------------
*/

IWORD get_owner_id        (UINT,UINT*                                        );
IWORD change_tree_father  (UINT,UINT                                         );
IWORD change_object_owner (UINT,UINT                                         );
IWORD get_owner_tree      (UINT,UINT                                         );
IWORD get_owner_father    (UINT,UINT*                                        );
IWORD get_owner_brothers  (UINT,UINT[],UINT,UINT*                            );
IWORD get_owner_sons      (UINT,UINT[],UINT,UINT*                            );
IWORD search_tree_path    (UINT,UINT,UINT[],UINT,UINT*                       );

/*
    4.10 Procedures prototypes for miscelleanous
    --------------------------------------------
*/

IWORD set_idle            (UINT                                              );
IWORD trerror             (INT                                               );
IWORD trempty             (INT                                               );
void  get_autvl           (INT*,INT*                                         );
void  get_vmk_stats       (UINT*,UINT*,UINT*                                 );
void  clear_vmk_stats     (void                                              );
void  set_putevt_vmk_err  (void(*)(),INT,INT                                 );
IWORD set_icc             (void                                              );


/*
    4.11 Procedures prototypes for software debugger
    ------------------------------------------------
*/

IWORD set_vhalt_att       (UINT,INT                                          );
void  vhalt               (IWORD                                             );
void  set_evt_debug       (S_evt*                                            );
IWORD chk_evt_debug       (S_evt*                                            );


/*
    4.12 Procedures prototypes for VMK lower layer
    ----------------------------------------------
*/

IWORD main                (INT,CHAR*[]                                       );
IWORD putevt_fae          (S_evt*                                            );
IWORD init_hypsys         (void                                              );
void  end_hypsys          (void                                              );
IWORD hypsys_wait         (INT,INT,S_evt*                                    );
IWORD get_board_name      (CHAR*                                             );

IWORD init_vmk_spe        (void                                              );
IWORD init_stack          (IWORD                                             );
IWORD init_task           (IWORD,IWORD                                       );
IWORD exec_task           (IWORD                                             );
IWORD return_task         (IWORD,IWORD                                       );
IWORD end_task            (IWORD                                             );
IWORD end_stack           (IWORD                                             );
IWORD vm_sleep            (IWORD                                             );


#endif                                 /* End of VMK_H                       */

/*
    5 RAM data to be included in space_appcont_xyz.c
    ------------------------------------------------
*/

#ifdef SPACE_APPCONT_XYZ_DATA_C

/*  5.1  FSM's
    ----------
*/

Aut_ctx      vmk_aut_ctx[VMK_AUT_MAX]             ;
const UCHAR  vmk_aut_max = VMK_AUT_MAX            ;


/*  5.2  VMK internal queues
    ------------------------
*/

S_evt        vmk_iq_evt  [VMK_IQ_EVT_MAX]          ;
UCHAR        vmk_evt_lock[VMK_IQ_EVT_MAX]          ;
const USHORT vmk_iq_evt_max = VMK_IQ_EVT_MAX       ;


/*  5.3  VMK routes
    ----------------
*/

Route        vmk_route[VMK_ROUTE_MAX]              ;
const UCHAR  vmk_route_max = VMK_ROUTE_MAX         ;

Sess_filt    vmk_sess_filt[VMK_SESS_COND_MAX]      ;
const USHORT vmk_sess_filt_max = VMK_SESS_COND_MAX ;


/*  5.4  VMK timers
    ---------------
*/

S_to         vmk_to[VMK_TO_MAX]                    ;
const UCHAR  vmk_to_max = VMK_TO_MAX               ;


/*  5.5  VMK tasks
    --------------
*/

Task_system  task_sys_ctx[VMK_TASK_MAX  ]          ;
Task_ctx     vmk_task[VMK_TASK_MAX]                ;
USHORT       task_state[VMK_TASK_MAX]              ;
UCHAR        task_pstack[VMK_TASK_MAX]             ;
const UCHAR  vmk_task_max = VMK_TASK_MAX           ;


/*  5.6  VMK semaphores
    -------------------
*/

Task_sem     vmk_sem[VMK_SEM_MAX]                  ;
const UCHAR  vmk_sem_max = VMK_SEM_MAX             ;


/*  5.7  VMK owners
    --------------
*/

S_owner      vmk_owner[VMK_OWNER_MAX]              ;
const UCHAR  vmk_owner_max = VMK_OWNER_MAX         ;

/*  5.8  VMK stacks
    ---------------
*/
const UCHAR nb_stack_sz=NB_STACK_SZ  ; /* Count of stack sizes               */

UINT        used_stack  [NB_STACK_SZ]; /* Stacks allocation bitmaps          */

const USHORT stack_sz[] =              /* Table des tailles de piles         */
              BEGIN                    /* (unite: 1 Ko).                     */
                STACK_SZ0,             /*                                    */
                STACK_SZ1,             /*                                    */
                STACK_SZ2,             /*                                    */
                STACK_SZ3,             /*                                    */
                STACK_SZ4              /*                                    */
              END                    ; /*                                    */
                                       /*                                    */
const USHORT stack_nb[] =              /* Table des nombres de piles         */
              BEGIN                    /* de chaque taille.                  */
                STACK_NB0            , /*                                    */
                STACK_NB1            , /*                                    */
                STACK_NB2            , /*                                    */
                STACK_NB3            , /*                                    */
                STACK_NB4              /*                                    */
              END                    ; /*                                    */
                                       /*                                    */
INT          stack [1024 * (           /* L'ensembles des piles              */
                STACK_SZ0*STACK_NB0  + /* allouables, alignees.              */
                STACK_SZ1*STACK_NB1  + /*                                    */
                STACK_SZ2*STACK_NB2  + /*                                    */
                STACK_SZ3*STACK_NB3  + /*                                    */
                STACK_SZ4*STACK_NB4)/ZI]; /*                                 */

#endif                                 /* End of APP_CONT_XYZ_DATA_C          */
