/*
**  helloim3.c - Sample code for HyperpanelOS =============================  **
**                                                                           **
**  This simple code is located into the Application Container, it is run    **
**  by the VMK sub-operating system. On the other hand, the I/O container    **
**  runs all the drivers that are VMIO Finite State machines.                **
**                                                                           **
**  Dedicated to STM MB1136 Cortex-M3 evaluation bord. The goal of this      **
**  small aplication is :                                                    **
**                                                                           **
**  -  To write a single message "Hello World" each second to serial port.   **
**  -  To respond to press / release of user button (blue button)            **
**                                                                           **
**  =======================================================================  **
*/

/* Includes files and external references ...................................*/

#include <hypos.h>                     // Hyperpanel OS basic interfaces
#include <drv_asy.h>                   // Prototype of "asy_write"
#include <tsk_iolib.h>                 // vmk/lib/tsk_iolib.h interface

/* Internal defines of this module ------------------------------------------*/

#define  TICK  10000                   // Code for tick event

/* Internal global variables of this module ---------------------------------*/
 
unsigned int   idto                  ; // Timer identifier
static int     gpio_iod              ; // IOD of device GPIO       
static int     gpio_iod1             ; // IOD of subchannel 0 of GPIO

static const char *gpio_taglist      = // USER button configuration
                                       //
                  "FUNC=GPIO\n"        // Button USER is on PC13 gpio in
                  "EVENTS=REPORT\n"    // Ask the driver to generate IND_REPORT
                  "MONITOR=IRQ"      ; // Pin state monitored under interrupt

static const char *bstate[]          = // For a trace when USER button is used
                    { "press"      ,   // Button is pressed
                      "release"    } ; // Button is released

/*  Prototypes --------------------------------------------------------------*/

    static int  wait_evt(void)       ; // waitevt_task() encapsulation

/* Beginning of the code ----------------------------------------------------*/


/*  Procedure loop_app_tsk ----------------------------------------------------

    Purpose : This is our task main loop. This is the entry point for
              create_task.
*/

int loop_app_tsk (void *param)
  {
    int             ev               ; // Our event
    int             cpt = 0          ; // Counter of messages
    char            mess[80]         ; // Message to be sent

/*****************************************************************************
 * Step 1 : Initialisation of the GPIO used for the USER button.             *
 * ------                                                                    *
 *****************************************************************************/

    asy_write(0,(unsigned char*)"Init ...\r\n",10); // Send a trace

    io_open("GPIO","GPIO",""         , // Open GPIO driver
                   &gpio_iod)        ; // 

    io_alloc_sub(gpio_iod            , // Alloc USER button GPIO sub-channel
                   "UNAME=USER"      , //
                   &gpio_iod1       ); //

    if( gpio_iod1 NE -1)               // If GPIO sub-channel is available,
      io_setval(gpio_iod1            , // configure the GPIO.
                gpio_taglist)        ; // 

    asy_write(0,(unsigned char*)"... done\r\n",10); // Send a trace         

/*****************************************************************************
 * Step 2 : We start a timer that will send us an event every 1 second so    *
 * ------   that we can update the time.                                     *
 *****************************************************************************/

    set_uto(CLOCK      ,               // Timer mode: clock
            1000       ,               // Duration in milliseconds
            TICK       ,               // Event code
            0          ,               // Event reserve field
            &idto        )           ; // Timer identifier


/*****************************************************************************
 * Step 3 : Here is our main event loop. For each loop, we do as follows :   *
 * -------  - First we wait for an event.                                    *
 *          - Then if the event code is TICK we print "Hello World"          *
 *          - On other cases the event is the USER button                    *
 *****************************************************************************/

    wait_ev                          : // Beginning of loop label

    ev = wait_evt()                  ; // Unschedule until an event is
                                       // received
    if ( ev == TICK )                  // If the event is the tick event
      {                                //
        hsprintf(mess            ,     // We build in "mess" the message
           "%6d  Hello World\r\n",     // to be sent
            cpt ++                 ) ; //

        asy_write(0              ,     // We send on ASY1
            (unsigned char*)mess ,     // the "mess" message
            strlen(mess)           ) ; // Count of bytes to be sent
      }
    else                               // The event is IND_REPORT from
      {                                // the USER button
        hsprintf(mess                , // We build in "mess" the message
        "%6d  User button <%s>\r\n"  , // to be sent
          cpt ++                     , // task_evt.length = 0 -> press
          bstate[task_evt.length]   ); //                   1 -> release

        asy_write(0              ,     // We send on ASY1
            (unsigned char*)mess ,     // the "mess" message
            strlen(mess)           ) ; // Count of bytes to be sent

      }

    goto wait_ev                     ; // Wait for the next event

    return 0                         ;
  }

/*  Procedure wait_evt ------------------------------------------------------*/
/*
    Purpose : Unschedule until the next event is received, whatever it is.
*/

static int wait_evt (void)
  {
    unsigned int waitlist[1][3]      ; // Parameter of "waitevt_task"

/*****************************************************************************
 * Step 1 : Build a list with one WAIT_CODEINT entry that will accept all    *
 * ------   the event codes ranging from 0 to 20000. Then call               *
 *          "waitevt_task", we will be unscheduled until the next event will *
 *          be received                                                      *
 *****************************************************************************/

    waitlist[0][0]  = WAIT_CODEINT   ; // All events with
    waitlist[0][1]  = 0              ; // a code between 0
    waitlist[0][2]  = 20000          ; // and 20000

    waitevt_task(waitlist ,            // Address of waiting list
                 1        ,            // Size of "waitlist[]"
                 0        ,            // maximum waiting time = no
                 0        )          ; // Do not purge previous events

/*****************************************************************************
 * Step 2 : Here we are scheduled again. The VMK has written into its        *
 * ------   global variable "task_evt" a copy of the event that has          *
 *          scheduled us again.                                              *
 *****************************************************************************/

    return task_evt.code             ; // Return event code
  }
